/*
    BFM, the Brutal File Manager. A 3D fps filemanager
    Copyright (C) 2003  Henrik Johansson and Daniel Forchheimer

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import java.awt.*;
import javax.swing.*;
import com.sun.j3d.utils.universe.*;
import javax.media.j3d.*;
import java.io.*;
import java.awt.event.*;
import java.util.*;

public class Bfm extends Frame implements ComponentListener, KeyListener, MouseMotionListener, MouseListener, MouseWheelListener, ActionListener {
	private SimpleUniverse universe;
	private Room room;
	private boolean playerHasPaused= false, showThumbnails= true, showBlood= true, autoChangeRooms= false, noTitleBar= false, pause=false, invertMouse= false, fullscreen=false, safeMode=false;
	private Canvas3D c3d;
	private BranchGroup bg;
	private Rectangle geometry;
	private String path,
	               pauseKey= "P",
	               strafeLeftKey= "A",
	               strafeRightKey= "D",
	               forwardKey= "W",
	               backwardKey= "S",
	               fullscreenKey= "F",
	               menuKey= Toolkit.getProperty("AWT.escape", "Escape"),
	               jumpKey= Toolkit.getProperty("AWT.space", "Space"),
	               crouchKey= Toolkit.getProperty("AWT.control", "Control"),
	               runKey= Toolkit.getProperty("AWT.shift", "Shift"),
	               toggleRunKey= Toolkit.getProperty("AWT.capsLock", "Caps Lock"),
	               removeModeKey= Toolkit.getProperty("AWT.f1", "F1"),
	               selectModeKey= Toolkit.getProperty("AWT.f2", "F2"),
	               unselectModeKey= Toolkit.getProperty("AWT.f3", "F3"),
								 deselectAllKey= Toolkit.getProperty("AWT.f7", "F7"),
	               copyKey= Toolkit.getProperty("AWT.f5", "F5"),
	               moveKey= Toolkit.getProperty("AWT.f6", "F6"),
								 toggleModeKey= Toolkit.getProperty("AWT.f4", "F4"),
								 useKey= "E",
								 teleportKey= "T",
	               fireKey= "mouse1",
	               secondaryFireKey= "mouse3",
	               prevWeaponKey= "mousewheelDown",
	               nextWeaponKey= "mousewheelUp",
								 helpMessageKey= "H",
								 statsKey= Toolkit.getProperty("AWT.tab", "Tab");
	private double mouseSensitivity= 6, mouseScale= 0.001, fieldOfView= 0.785;
	private Player player;
	private Insets insets;
	private Panel contentPanel, canvasCard;
	private OpenDialog teleportPanel;
	private PhysicsHandler physicsHandler;
	private int modeIndex=0, helpMessageIndex=0, resX= (int)Toolkit.getDefaultToolkit().getScreenSize().getWidth(), resY= (int)Toolkit.getDefaultToolkit().getScreenSize().getHeight(), sizeX=resX/2, sizeY=resY/2, posX=(resX-sizeX)/2, posY=(resY-sizeY)/2, oldMousePosX=0, oldMousePosY=0, mouseDiffX=0, mouseDiffY=0, mouseMargin=100;
	private final String CANVASCARD="canvasCard", MENUCARD="menuCard", TELEPORTPANEL="teleportPanel", ABOUTPANEL="aboutPanel", BLANKPANEL="blankPanel", QUITBUTTONTEXT="Quit", ABOUTBUTTONTEXT="About", RESUMEBUTTONTEXT="Resume", TELEPORTBUTTONTEXT="Teleporter", NAME="BFM", LONGNAME="The Brutal File Manager", VERSION="1.2", RELEASEDATE="22nd of January 2005", TITLE= "["+NAME+" v"+VERSION+"]", ABOUTHTML= "<center>"+NAME+", "+LONGNAME+" v"+VERSION+"<br><br>Created by Henrik Johansson<br>and Daniel Forchheimer<br><br>Take a look at <br>http://www.forchheimer.se/bfm<br> for more information<br><br>"+NAME+" is released under the <br>GNU General Public Licence<br>http://www.gnu.org";
	private String[] weaponKeys= { "1", "2", "3", "4", "5"/*, "6"*/ },
	                 modes= { "Remove", "Select", "Unselect" },
	                 fortune= { "hej tangent",
	                            "Hlldre fel kod n ingen kod",
	                            "we have no coding standard",
	                            "Apanap loves to remove files with BFM!",
	                            "an entertaining way to remove your files",
	                            "I throw again, giving to the customer rope in order to hang itself enough" };

	public static void main( String[] args ) { new Bfm( args ); }

	private Bfm(String[] args) {
		/* set title */
		super.setTitle( TITLE+" - \""+ fortune[new Random().nextInt(fortune.length)]+"\"" );
		setLayout( new CardLayout() );
		Panel menuCard= createInfoPanel( ABOUTHTML );
		canvasCard= new Panel();
		canvasCard.setLayout( new BorderLayout() );
		add( menuCard, MENUCARD );
		add( canvasCard, CANVASCARD );

		/* read config files and command line options, don't bother to try{} catch() {}, readConfigFile will take care of that*/
		readConfigFile( "/etc/bfm/bfm.conf" );
		readConfigFile( System.getProperty( "user.home" )+"/.bfm/bfm.conf" );
		commandLineParser(args);

		/* create canvas and universe */
		c3d= new Canvas3D( SimpleUniverse.getPreferredConfiguration() );
		canvasCard.add(c3d);
		bg= new BranchGroup();
		bg.setCapability(Group.ALLOW_CHILDREN_READ);
		bg.setCapability(Group.ALLOW_CHILDREN_WRITE);
		bg.setCapability(Group.ALLOW_CHILDREN_EXTEND);
		universe= new SimpleUniverse(c3d);
		universe.getViewer().getView().setBackClipDistance(50.0);
		universe.getViewer().getView().setFrontClipDistance(.1f);
		universe.getViewer().getView().setTransparencySortingPolicy(View.TRANSPARENCY_SORT_GEOMETRY);
		universe.getViewer().getView().setFrontClipPolicy(View.VIRTUAL_EYE);
		universe.getViewer().getView().setFieldOfView(fieldOfView);
		universe.addBranchGraph(bg);
		
		/* misc settings, show Frame */
		insets= this.getInsets();
		show();
		setFocusTraversalKeys(KeyboardFocusManager.FORWARD_TRAVERSAL_KEYS,Collections.EMPTY_SET);
		setBounds( posX, posY, sizeX, sizeY );
		hideCursor();
		if( noTitleBar ) this.setUndecorated( true );
		if( fullscreen ) setFullscreen();

		/* add mouse, keyboard, component and window listeners, show everything */
		this.addComponentListener( this );
		this.addWindowListener( new WindowAdapter() { public void windowClosing(WindowEvent e) { System.exit(0); } } );
		c3d.addKeyListener( this );
		c3d.addMouseMotionListener( this );
		c3d.addMouseListener( this );
		c3d.addMouseWheelListener( this );
		startGame();
		resetMouse();
		((CardLayout)getLayout()).show( this, CANVASCARD );
		remove( menuCard );
		add( createMenu(), MENUCARD );
		c3d.requestFocus();
	}

	private void startGame() {
		/* create Player, PhysicsHandler and Room */
		player= new Player( universe, fieldOfView );
		updateHudPath();
		if( safeMode ) player.setMode( "Safe" );
		else player.setMode( player.getMode() );
		room= new Room( path, this );
		bg.addChild(room);
		physicsHandler= new PhysicsHandler(player,this);
	}
	
	/* read settings from config files */
	private void readConfigFile( String path ) {
		try {
			String line="";
			BufferedReader in= new BufferedReader( new FileReader( path ));
			while(( line=in.readLine().replaceAll( " ", "" ).replaceAll( "\t", "" ) ) != null ) {
				if( !line.startsWith("#") ) {
					     if( line.startsWith( "sizeX=" )) sizeX= Integer.parseInt( line.substring( line.indexOf("=")+1 ));
					else if( line.startsWith( "sizeY=" )) sizeY= Integer.parseInt( line.substring( line.indexOf("=")+1 ));
					else if( line.startsWith( "posX=" )) posX= Integer.parseInt( line.substring( line.indexOf("=")+1 ));
					else if( line.startsWith( "posY=" )) posY= Integer.parseInt( line.substring( line.indexOf("=")+1 ));
					else if( line.startsWith( "safeMode=" )) safeMode= Boolean.valueOf( line.substring( line.indexOf("=")+1 )).booleanValue();
					else if( line.startsWith( "fullscreen=" )) fullscreen= Boolean.valueOf( line.substring( line.indexOf("=")+1 )).booleanValue();
					else if( line.startsWith( "invertMouse=" )) invertMouse= Boolean.valueOf( line.substring( line.indexOf("=")+1 )).booleanValue();
					else if( line.startsWith( "noTitleBar=" )) noTitleBar= Boolean.valueOf( line.substring( line.indexOf("=")+1 )).booleanValue();
					else if( line.startsWith( "showBlood=" )) showBlood= Boolean.valueOf( line.substring( line.indexOf("=")+1 )).booleanValue();
					else if( line.startsWith( "showThumbnails=" )) showThumbnails= Boolean.valueOf( line.substring( line.indexOf("=")+1 )).booleanValue();
					else if( line.startsWith( "autoChangeRooms=" )) autoChangeRooms= Boolean.valueOf( line.substring( line.indexOf("=")+1 )).booleanValue();
					else if( line.startsWith( "forward=" )) { line= line.substring( line.indexOf("=")+1 ); forwardKey= Toolkit.getProperty("AWT."+line, line); }
					else if( line.startsWith( "backward=" )) { line= line.substring( line.indexOf("=")+1 ); backwardKey= Toolkit.getProperty("AWT."+line, line); }
					else if( line.startsWith( "strafeLeft=" )) { line= line.substring( line.indexOf("=")+1 ); strafeLeftKey= Toolkit.getProperty("AWT."+line, line); }
					else if( line.startsWith( "strafeRight=" )) { line= line.substring( line.indexOf("=")+1 ); strafeRightKey= Toolkit.getProperty("AWT."+line, line); }
					else if( line.startsWith( "jump=" )) { line= line.substring( line.indexOf("=")+1 ); jumpKey= Toolkit.getProperty("AWT."+line, line); }
					else if( line.startsWith( "crouch=" )) { line= line.substring( line.indexOf("=")+1 ); crouchKey= Toolkit.getProperty("AWT."+line, line); }
					else if( line.startsWith( "run=" )) { line= line.substring( line.indexOf("=")+1 ); runKey= Toolkit.getProperty("AWT."+line, line); }
					else if( line.startsWith( "toggleRun=" )) { line= line.substring( line.indexOf("=")+1 ); toggleRunKey= Toolkit.getProperty("AWT."+line, line); }
					else if( line.startsWith( "pause=" )) { line= line.substring( line.indexOf("=")+1 ); pauseKey= Toolkit.getProperty("AWT."+line, line); }
					else if( line.startsWith( "fullscreen=" )) { line= line.substring( line.indexOf("=")+1 ); fullscreenKey= Toolkit.getProperty("AWT."+line, line); }
					else if( line.startsWith( "menu=" )) { line= line.substring( line.indexOf("=")+1 ); menuKey= Toolkit.getProperty("AWT."+line, line); }
					else if( line.startsWith( "helpMessage=" )) { line= line.substring( line.indexOf("=")+1 ); helpMessageKey= Toolkit.getProperty("AWT."+line, line); }
					else if( line.startsWith( "stats=" )) { line= line.substring( line.indexOf("=")+1 ); statsKey= Toolkit.getProperty("AWT."+line, line); }
					else if( line.startsWith( "use=" )) { line= line.substring( line.indexOf("=")+1 ); useKey= Toolkit.getProperty("AWT."+line, line); }
					else if( line.startsWith( "teleport=" )) { line= line.substring( line.indexOf("=")+1 ); teleportKey= Toolkit.getProperty("AWT."+line, line); }
					else if( line.startsWith( "mouseSensitivity=" )) { mouseSensitivity= Double.parseDouble( line.substring( line.indexOf("=")+1 )); }
					else if( line.startsWith( "fire=" )) { line= line.substring( line.indexOf("=")+1 ); fireKey= Toolkit.getProperty("AWT."+line, line); }
					else if( line.startsWith( "secondaryFire=" )) { line= line.substring( line.indexOf("=")+1 ); secondaryFireKey= Toolkit.getProperty("AWT."+line, line); }
					else if( line.startsWith( "nextWeapon=" )) { line= line.substring( line.indexOf("=")+1 ); nextWeaponKey= Toolkit.getProperty("AWT."+line, line); }
					else if( line.startsWith( "prevWeapon=" )) { line= line.substring( line.indexOf("=")+1 ); prevWeaponKey= Toolkit.getProperty("AWT."+line, line); }
					else if( line.startsWith( "removeMode=" )) { line= line.substring( line.indexOf("=")+1 ); removeModeKey= Toolkit.getProperty("AWT."+line, line); }
					else if( line.startsWith( "unselectMode=" )) { line= line.substring( line.indexOf("=")+1 ); unselectModeKey= Toolkit.getProperty("AWT."+line, line); }
					else if( line.startsWith( "deselectAll=" )) { line= line.substring( line.indexOf("=")+1 ); deselectAllKey= Toolkit.getProperty("AWT."+line, line); }
					else if( line.startsWith( "selectMode=" )) { line= line.substring( line.indexOf("=")+1 ); selectModeKey= Toolkit.getProperty("AWT."+line, line); }
					else if( line.startsWith( "selectMode=" )) { line= line.substring( line.indexOf("=")+1 ); toggleModeKey= Toolkit.getProperty("AWT."+line, line); }
					else if( line.startsWith( "copy=" )) { line= line.substring( line.indexOf("=")+1 ); copyKey= Toolkit.getProperty("AWT."+line, line); }
					else if( line.startsWith( "move=" )) { line= line.substring( line.indexOf("=")+1 ); moveKey= Toolkit.getProperty("AWT."+line, line); }
					else if( line.startsWith( "weapon" )) {
						int weapon= Integer.parseInt( line.substring( line.indexOf( "n" )+1, line.indexOf("=") ))-1;
						line= line.substring( line.indexOf("=")+1 );
						weaponKeys[weapon]= Toolkit.getProperty("AWT."+line, line);
					}
				}
			}
			in.close();
		} catch( Exception e ) {}
	}

	/* parse command line options */
	private void commandLineParser( String[] args ) {
		for(int i=0;i<args.length;i++) {
			if(args[i].equals("-l")||args[i].equals("--loadconfig")) {
				if(args.length>i+1) {
					readConfigFile( args[i+1] );
				} else {
					System.out.println(args[i]+" given without path");
					System.exit(-1);
				}
			}
		}
		for( int i=0; i<args.length; i++ ) {
			     if( args[i].equals( "-h" ) || args[i].equals( "--help") || args[i].equals( "-?" )) helpMessage();
			else if( args[i].equals( "-s" ) || args[i].equals( "--safemode" )) safeMode= true;
			else if( args[i].equals( "-u" ) || args[i].equals( "--unsafemode" )) safeMode= false;
			else if( args[i].equals( "-v" ) || args[i].equals( "--version" )) { System.out.println(NAME+", "+LONGNAME+" "+ VERSION+ " - released "+ RELEASEDATE); System.exit(0); }
			else if( args[i].equals( "-f" ) || args[i].equals( "--fullscreen" )) fullscreen= true;
			else if( args[i].equals( "-i" ) || args[i].equals( "--invertmouse" )) invertMouse= true;
			else if( args[i].equals( "-l" ) || args[i].equals( "--loadconfig" )) i++;
			else if( args[i].equals( "-c" ) || args[i].equals( "--config" )) usageMessage();
			else if( args[i].equals( "-n" ) || args[i].equals( "--noblood" )) showBlood= false;
			else {
				File f= new File( args[i] );
				try{
					path=f.getCanonicalPath();
				} catch( Exception e ) {
					path=f.getAbsolutePath().toString();	
				}
			}
		}
		if(path == null) {
			try {
				path= System.getProperty("user.dir");	
			} catch (SecurityException se) {
				System.out.println("pwd failed, defualt is user home dir");
				path="user.home";
			}
		} 
	}

	/* hide cursor when unpaused */
	public void hideCursor() {
		Image cursorImage= Toolkit.getDefaultToolkit().getImage("cursor.gif");
		MediaTracker mediaTracker= new MediaTracker(this);
		mediaTracker.addImage(cursorImage, 0);
		Cursor cursor= Toolkit.getDefaultToolkit().createCustomCursor(cursorImage, new Point(0,0), "cursor");
		setCursor(cursor);
	}

	private void showCursor() {
		setCursor( Cursor.getPredefinedCursor( Cursor.DEFAULT_CURSOR ));
	}

	/* toggle fullscreen */
	private void toggleFullscreen() {
		if( !fullscreen ) {
			setFullscreen();
		} else if( fullscreen ) {
			fullscreen= false;
			setBounds( geometry.x+insets.left, geometry.y+insets.top, geometry.width, geometry.height );
		}
	}

	private void setFullscreen() {
		fullscreen= true;
		geometry=this.getBounds();
		setBounds( 0, 0, resX+insets.left+insets.right, resY+insets.top+insets.bottom );
	}

	/* misc get/set methods */
	public Player getPlayer() { return player; }
	public boolean getPause() { return pause; }
	public boolean getAutoChangeRooms() { return autoChangeRooms; }
	public boolean getShowBlood() { return showBlood; }
	public boolean getShowThumbnails() { return showThumbnails; }
	public SimpleUniverse getUniverse() { return universe; }
	public boolean getSafeMode() { return safeMode; } 

	/* pause */
	private void togglePause() {
		if( !pause ) {
			playerHasPaused= true;
			pause=true;
			if( player.isAlive() ) player.getHud().setCurrentDirText("Paused, press "+pauseKey+" to continue");
			showCursor();
		} else if( pause ) {
			playerHasPaused= false;
			hideCursor();
			resetMouse();
			if( player.isAlive() ) {
				resume();
				updateHudPath();
			}
		}
	}

	/* get/set room */
	public Room getRoom() { return room; }
	public Room setNewRoom( String path ) {
		this.path= path;
		room.detach();
		
		/* finish Rockets and DroppedDynamite in Room */
		for( int i=0; i<room.numChildren(); i++ ) {
			try{ ((Rocket)(room.getChild(i))).finish(); } catch( Exception e ) {}
			try{ ((DroppedDynamite)(room.getChild(i))).finish(); } catch( Exception e ) {}
		}
		for( int i=0; i<room.numChildren(); i++ ) {
			try {
				room.removeChild( (FileObject)(room.getChild(i)) );
			} catch( Exception e  ) {}
		}

		room= new Room( path, this );
		bg.addChild(room);
		updateHudPath();
		player.setAlive( true );
		return room;
	}

	/* print help message and exit */
	private void helpMessage() {
		                  /*--width of 80 character terminal-----------------------------------------------> */
		System.out.println(NAME+", "+LONGNAME+" "+ VERSION+ " - released "+ RELEASEDATE);
		System.out.println("");
		System.out.println("Usage: bfm [PATH] [OPTIONS]");
		System.out.println("Delete your files in an brutal and entertaining way...");
		System.out.println("Options:");
		System.out.println("  -c, --config           print the current configuration and exit");
		System.out.println("  -f, --fullscreen       start in fullscreen mode");
		System.out.println("  -h, --help             prints this message");
		System.out.println("  -i, --invertmouse      invert mouse");
		System.out.println("  -l, --loadconfig=PATH  load config file from path: 'bfm -l c:\\bfm\\bfm.conf'");
		System.out.println("  -n  --noblood          don't display blood");
		System.out.println("  -s, --safemode         safe mode, no files will actually be deleted");
		System.out.println("  -u, --unsafemode       delete files, overrides safe mode set in config file");
		System.out.println("  -v, --version          print version and quit");
		System.out.println("");
		System.out.println("Authors: Daniel Forchheimer and Henrik Johansson");
		System.exit(0);
	}

	/* print current configuration */
	private void usageMessage() {
		System.out.println("This is the current configuration:\n");
		for( int o=0; o<2; o++ ) {
			String[][] message= getHelpMessage();
			for( int i=0; i<message.length; i++ ) {
				if( message[i][0].indexOf( "Press" )==-1 ) {
					System.out.print( message[i][0] );
					for( int n=0; n<25-message[i][0].length(); n++ ) System.out.print( " " );
					System.out.println( message[i][1] );
				}
			}
		}
		System.out.println("All keys can be changed by editing bfm.conf");
		System.exit(0);
	}

	/* move cursor to center of Canvas3D*/
	public void resetMouse() {
		try {
			Robot robot= new Robot();
			robot.mouseMove(getBounds().x+this.getWidth()/2,getBounds().y+this.getHeight()/2);
		} catch ( Exception e ) { System.out.println( "unable to create Robot" ); }
		oldMousePosX=this.getWidth()/2;
		oldMousePosY=this.getHeight()/2;
	}

	public void changeMode( String mode ) {
		player.setMode( mode );
		for( int i=0; i<room.numChildren(); i++ ) {
			try {
				((FileObject)(room.getChild(i))).hit(0);
			} catch( Exception e ) {}
		}
		for( int i=0; i<room.getWalls().length; i++ ) room.getWalls()[i].update( player.getMode() );
	}

	/* componentlistener stuff */	
	public void componentHidden(ComponentEvent ce) {}
	public void componentMoved(ComponentEvent ce) { resetMouse(); }
	public void componentResized(ComponentEvent ce) { resetMouse(); player.getHud().refresh( universe.getCanvas().getSize(), universe.getViewer().getView().getFieldOfView(), fieldOfView ); }
	public void componentShown(ComponentEvent ce) {}

	/* keylistener stuff */
	public void keyPressed( KeyEvent ke ) { pressedEvent( ke.getKeyText( ke.getKeyCode() )); }
	public void keyReleased( KeyEvent ke ) { releasedEvent( ke.getKeyText( ke.getKeyCode() )); }
	public void keyTyped( KeyEvent ke ) {}

	/* mouselistener stuff */
	public void mouseClicked( MouseEvent me ) {}
	public void mouseEntered( MouseEvent me ) {}
	public void mouseExited( MouseEvent me ) {}
	public void mousePressed( MouseEvent me ) { pressedEvent( "mouse"+me.getButton() ); }
	public void mouseReleased( MouseEvent me ) { releasedEvent( "mouse"+me.getButton() ); }

	/* mousewheellistener stuff */
	public void mouseWheelMoved( MouseWheelEvent mwe ) {
		     if( mwe.getWheelRotation() == -1 ) pressedEvent( "mousewheelUp" );
		else if( mwe.getWheelRotation() == 1 ) pressedEvent( "mousewheelDown" );
	}

	/* mousemotionlistener stuff */
	public void mouseDragged( MouseEvent me ) { mouseMoved( me ); }
	public void mouseMoved( MouseEvent me ) {
		if( !pause ) {
			mouseDiffX= oldMousePosX - me.getX();
			mouseDiffY= oldMousePosY - me.getY();
			oldMousePosX= me.getX();
			oldMousePosY= me.getY();
			if( invertMouse ) player.lookUp( mouseDiffY*mouseSensitivity*mouseScale*-1 );
			else player.lookUp( mouseDiffY*mouseSensitivity*mouseScale );
			player.turnRight( mouseDiffX*mouseSensitivity*mouseScale );
			if( me.getX()<mouseMargin || me.getX()>this.getWidth()-mouseMargin || me.getY()<mouseMargin || me.getY()>this.getHeight()-mouseMargin ) {
				resetMouse();
			}
		}
	}

	/* actionlistener stuff */
	public void actionPerformed( ActionEvent ae ) {
		String cmd= ae.getActionCommand();
		     if( cmd.equals( QUITBUTTONTEXT )) System.exit( 0 );
		else if( cmd.equals( ABOUTBUTTONTEXT )) ((CardLayout)contentPanel.getLayout()).show( contentPanel, ABOUTPANEL );
		else if( cmd.equals( RESUMEBUTTONTEXT )) toggleMenu();
		else if( cmd.equals( TELEPORTBUTTONTEXT )) showTeleportPanel();
	}

	/* pressed events for mouse and keyboard */
	private void pressedEvent( String event ) {
		     if( event.equals( pauseKey )) togglePause();
		else if( event.equals( fullscreenKey )) toggleFullscreen();
		/*else if( event.equals( helpMessageKey )) {
			if( !player.getHud().getMessageType().equals( "help" )) helpMessageIndex= 0;
			if( helpMessageIndex==1 && player.getHud().getMessageType().equals( "help" )) player.getHud().setMessage( getHelpMessage() );
			else player.getHud().toggleMessage( getHelpMessage(), "help" );
		}*/
		else if( event.equals( teleportKey )) { showCursor(); pause= true; ((CardLayout)getLayout()).show( this, MENUCARD ); showTeleportPanel(); }
		else if( event.equals( statsKey )) { player.getHud().toggleMessage( player.getStats(), "stats" ); }
		else if( event.equals( menuKey )) toggleMenu();
		else if( event.equals( strafeLeftKey )) player.setVelocityX( -player.speed );
		else if( event.equals( strafeRightKey )) player.setVelocityX( player.speed );
		else if( event.equals( forwardKey )) player.setVelocityZ( player.speed );
		else if( event.equals( backwardKey )) player.setVelocityZ( -player.speed );
		else if( event.equals( runKey )) player.changeSpeed();
		else if( event.equals( toggleRunKey )) player.toggleRun();
		else if( event.equals( jumpKey )) player.jump();
		else if( event.equals( crouchKey )) player.toggleCrouch();
		else if( event.equals( fireKey )) { if( player.getWeapon().getRepeatFire() ) player.fire= true; if( !player.isAlive() ) restart(); }
		else if( event.equals( secondaryFireKey )) { if( player.getWeapon().getRepeatSecondaryFire() ) player.secondaryFire= true; else player.getWeapon().secondaryFire( room ); }
		else if( !pause && player.isAlive() ) {
			/*     if( event.equals( removeModeKey ) && !safeMode ) { modeIndex=0; changeMode( modes[0] ); }
			else if( event.equals( selectModeKey ) && !safeMode ) { modeIndex=1; changeMode( modes[1] ); }
			else if( event.equals( unselectModeKey ) && !safeMode ) { modeIndex=2; changeMode( modes[2] ); }
			else if( event.equals( deselectAllKey ) && !safeMode ) player.resetSelectedFiles( room );
			else if( event.equals( toggleModeKey ) && !safeMode ) { modeIndex++; modeIndex= modeIndex%modes.length; changeMode( modes[modeIndex] ); }
			else if( event.equals( copyKey )) room.copyFiles( player.getSelectedFiles() );
			else if( event.equals( moveKey )) room.moveFiles( player.getSelectedFiles() );
			else*/ if( event.equals( nextWeaponKey )) player.nextWeapon( weaponKeys.length );
			else if( event.equals( prevWeaponKey )) player.prevWeapon( weaponKeys.length );
			else { for( int i=0; i<weaponKeys.length; i++ ) { if( event.equals( weaponKeys[i] )) player.setNewWeapon( i ); }}
		}
	}

	/* released events for mouse and keyboard  */
	private void releasedEvent( String event ) {
	       if( event.equals( strafeLeftKey )) player.setVelocityX(0);
		else if( event.equals( strafeRightKey )) player.setVelocityX(0);
		else if( event.equals( forwardKey )) player.setVelocityZ(0);
		else if( event.equals( backwardKey )) player.setVelocityZ(0);
		else if( event.equals( runKey )) player.changeSpeed();
		else if( event.equals( crouchKey )) player.toggleCrouch();
		else if( event.equals( useKey )) player.using= true;
		else if( event.equals( fireKey )) player.fire= false;
		else if( event.equals( secondaryFireKey )) player.secondaryFire= false;
	}

	private String[][] getHelpMessage() {
		helpMessageIndex++;
		helpMessageIndex= helpMessageIndex%3;
		if( helpMessageIndex==1 ) {
			String[][] commands= { { "Controls:", "" },
			                       { "Forward", forwardKey },
			                       { "Backward", backwardKey },
														 { "Strafe left", strafeLeftKey },
														 { "Strafe right", strafeRightKey },
														 { "Jump", jumpKey },
														 { "Crouch", crouchKey },
														 { "Run", runKey },
														 { "Toggle run on/off", toggleRunKey },
														 { "Use", useKey },
														 { "", "" },
														 { "Firing:", "" },
														 { "Spear", weaponKeys[0] },
														 { "Shotgun", weaponKeys[1] },
														 { "Sniper rifle", weaponKeys[2] },
														 { "Dynamite", weaponKeys[3] },
														 { "Rocket launcher", weaponKeys[4] },
														 { "Fire", fireKey },
														 { "Secondary fire", secondaryFireKey },
														 { "Next weapon", nextWeaponKey },
														 { "Previous weapon", prevWeaponKey },
														 { "", "" },
														 { "Press "+helpMessageKey+" again for more settings...", "" } };
			return commands;
		} else if( helpMessageIndex==2 ) {
			String[][] commands= { /*{ "File managing:", "" },
													   { "Remove mode", removeModeKey },
													   { "Select mode", selectModeKey },
		                         { "Unselect mode", unselectModeKey },
													   { "Toggle mode", toggleModeKey },
														 { "Copy", copyKey },
														 { "Move", moveKey },
														 { "Deselect all files", deselectAllKey },
													   { "", "" },*/
													   { "Misc:", "" },
													   { "Fullscreen", fullscreenKey },
													   { "Pause", pauseKey },
														 { "Display stats", statsKey },
													   //{ "Display help message", helpMessageKey },
                             { "Show menu", menuKey },
													   { "", "" },
		                         { "Settings:", "" },
		                         { "Safe mode", String.valueOf(safeMode) },
		                         { "Invert mouse", String.valueOf(invertMouse) },
		                         { "Fullscreen", String.valueOf(fullscreen) },
		                         { "Mouse sensitivity", String.valueOf(mouseSensitivity) },
		                         { "Auto change rooms", String.valueOf(autoChangeRooms) },
		                         { "Undecorated window", String.valueOf(noTitleBar) },
														 { "", "" },
														 { "Press "+helpMessageKey+" again to hide help message...", "" } };
			return commands;
		} else {
			return null;
		}
	}

	private void restart() {
		player.remove();
		bg.removeChild(room);
		startGame();
	}

	public void resume() {
		pause= false;
		physicsHandler.startThread();
	}

	private void updateHudPath() {
		player.getHud().setCurrentDirText("Path: "+path);
	}


	/* GUI section */
	private Panel createMenu() {
		Panel panel= new Panel( new BorderLayout() );
		panel.setBackground( Color.BLACK );
		
		Panel menu= new Panel();
		JButton[] menuItems= { new JButton( TELEPORTBUTTONTEXT ), new JButton( ABOUTBUTTONTEXT ), new JButton( RESUMEBUTTONTEXT ), new JButton( QUITBUTTONTEXT ) };
		for( int i=0; i<menuItems.length; i++ ) {
			menuItems[i].setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ));
			menuItems[i].setOpaque( false );
			menuItems[i].addActionListener( this );
			menuItems[i].setFont( new Font( "Arial", Font.BOLD, 12 ));
			menuItems[i].setForeground( Color.YELLOW );
			menu.add( menuItems[i] );
		}
		panel.add( menu, BorderLayout.PAGE_START );

		contentPanel= new Panel( new CardLayout() );
		Panel blank= new Panel();
		contentPanel.add( blank, BLANKPANEL );
		teleportPanel= new OpenDialog( this, TITLE, room.getPath() );
		contentPanel.add( teleportPanel, TELEPORTPANEL );
		Panel about= createInfoPanel( ABOUTHTML );
		contentPanel.add( about, ABOUTPANEL );
		panel.add( contentPanel );
		return panel;
	}

	private Panel createInfoPanel( String text ) {
		Panel panel= new Panel();
		panel.setBackground( Color.BLACK );
		try{
			JEditorPane textPane= new JEditorPane( "text/html", "<font face=\"arial\" style=\"color:yellow;font-weight:bold;font-size:10px;\">"+text );
			textPane.setEditable( false );
			textPane.setOpaque( false );
			JScrollPane scrollPane= new JScrollPane( textPane );
			scrollPane.setBorder( BorderFactory.createEmptyBorder( 0, 0, 0, 0 ));
			scrollPane.getViewport().setBackground( Color.BLACK );
			panel.add( scrollPane );
		} catch( Exception e ) {}
		return panel;
	}

		private void toggleMenu() {
		if( canvasCard.isVisible() ) {
			pause= true;
			showCursor();
			((CardLayout)getLayout()).show( this, MENUCARD );
		} else {
			((CardLayout)contentPanel.getLayout()).show( contentPanel, BLANKPANEL );
			if( !playerHasPaused ) {
				resetMouse();
				hideCursor();
				resume();
			}
			((CardLayout)getLayout()).show( this, CANVASCARD );
			c3d.requestFocus();
		}
	}

	private void showTeleportPanel() {
		teleportPanel.refresh( room.getPath() );
		((CardLayout)contentPanel.getLayout()).show( contentPanel, TELEPORTPANEL );
	}
}
