/*
    BFM, the Brutal File Manager. A 3D fps filemanager
    Copyright (C) 2003  Henrik Johansson and Daniel Forchheimer

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import com.sun.j3d.utils.geometry.*;
import javax.media.j3d.*;
import javax.vecmath.*;

public class Dynamite extends Weapon {
	private TransformGroup animateTrans;
	private float radius;
	private Vector3f position;
	private int redBarFrames= 10;

	public Dynamite( Player player ) {
		super( player );
		animationFrames= 20;
		fireFrames= 150;
		fireDelay= 1f;
		autoSetAmmo= false;
		repeatFire= true; repeatSecondaryFire= false;
		/*ammo=5;*/ unlimitedSecondaryFire= true; unlimitedFire= true;
	}

	public void initializeAnimate() {
		t3d= new Transform3D();
		animateTrans.getTransform(t3d);
		t3d.setTranslation( new Vector3f( 0, -.15f+.15f*((float)currentAnimationFrame/(float)animationFrames), 0 ));
		animateTrans.setTransform(t3d);
	}

	public TransformGroup getCrosshair() { return new TransformGroup(); }

	public Transform3D initialTransform() {
		t3d= new Transform3D();
		t3d.rotZ( Math.PI/14 );
		return t3d;
	}

	TransformGroup createShape() {
		float radius=0.02f, length=.4f, moveX=radius, moveY=-.3f, moveZ=-.5f;
		this.radius= radius;

		Appearance weaponAppearance= new Appearance();
		Material material= new Material();
		material.setEmissiveColor( new Color3f( .6f, .6f, .5f ));
		material.setDiffuseColor( new Color3f( .6f, .6f, .5f ));
		weaponAppearance.setMaterial( material );

		Cylinder left= new Cylinder( radius, length, weaponAppearance );
		t3d= new Transform3D();
		t3d.setTranslation( new Vector3f( -moveX, moveY, moveZ) );
		TransformGroup leftGroup= new TransformGroup( t3d );
		leftGroup.addChild( left );

		Cylinder center= new Cylinder( radius, length, weaponAppearance );
		t3d= new Transform3D();
		t3d.setTranslation( new Vector3f( 0, moveY, moveZ-(float)Math.sqrt((radius*2)*(radius*2)-radius*radius)));
		TransformGroup centerGroup= new TransformGroup( t3d );
		centerGroup.addChild( center );

		Cylinder right= new Cylinder( radius, length, weaponAppearance );
		t3d= new Transform3D();
		t3d.setTranslation( new Vector3f( moveX, moveY, moveZ) );
		TransformGroup rightGroup= new TransformGroup( t3d );
		rightGroup.addChild( right );
 		
		animateTrans= new TransformGroup( initialTransform() );
		animateTrans.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
		animateTrans.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		animateTrans.addChild(leftGroup);
		animateTrans.addChild(centerGroup);
		animateTrans.addChild(rightGroup);
		return animateTrans;
	}
	
	public void fireThisWeapon() {
		/* show progressbar if the player is pressing fireKey */
		if( stillFiring ) {
			player.getHud().setProgressBar( currentAnimationFrame, fireFrames-redBarFrames );
			/* add dynamite to Room */
			if( currentAnimationFrame == fireFrames-redBarFrames ) {
				/*ammo--;
				player.getHud().setAmmoText( Integer.toString( ammo ));*/
				position= player.getPositionVector();
				Transform3D t3d2= new Transform3D();
				if( player.standingCloseToWall ) {
					t3d2.setTranslation( position );
				} else {
					t3d2.rotZ( Math.PI/2 );
					t3d2.setTranslation( new Vector3f( position.x, radius, position.z ));
					for(int o=0;o<room.numChildren();o++) {
						try {
							if(((FileObject)room.getChild(o)).checkXZIntersect(position)) {
								t3d2.setTranslation( new Vector3f( position.x, ((FileObject)room.getChild(o)).getHeight()+radius, position.z ));
								break;
							}
						} catch( Exception e ) {}
					}
				}
				DroppedDynamite droppedDynamite= new DroppedDynamite( createShape(), room, t3d2, position );
				room.addChild( droppedDynamite );
			}
		} else {
			/* abort and hide progressbar if player doesn't press fireKey long enough */
			currentAnimationFrame= 7361;
			timer= 5298f;
			player.getHud().hideProgressBar();
		}
		if( currentAnimationFrame==fireFrames ) player.getHud().hideProgressBar();
	}

	public void secondaryFireThisWeapon() {}

	public void endAnimate() {
		t3d= new Transform3D();
		animateTrans.getTransform(t3d);
		t3d.setTranslation( new Vector3f( 0, -.15f*((float)currentAnimationFrame/(float)animationFrames), 0 ));
		animateTrans.setTransform(t3d);
	}
}
