/*
    BFM, the Brutal File Manager. A 3D fps filemanager
    Copyright (C) 2003  Henrik Johansson and Daniel Forchheimer

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import java.awt.*;
import com.sun.j3d.utils.geometry.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import java.io.*;

public class FileObject extends RoomObject {
	private Cylinder cylinder;
	private File file;
	private Transform3D /*textRotateT3D,*/ originalPositionT3D, moveT3D;
	private TransformGroup /*textRotateTrans,*/ originalPositionTrans, moveTrans;
	private Shape3D textShape;
	private boolean selected= false, bleeding= false, dead= false;
	private Bfm bfm;
	private Vector3f filePosition;
	protected float radius= 1, height, health, bloodOffset=.5f;
	private Color3f editableSpecularColor= new Color3f(0,0,1), 
	                editableDiffuseColor= new Color3f(0,0,.7f),
	                editableEmissiveColor= new Color3f(0,0,.2f),
	                unEditableSpecularColor= new Color3f(1,0,0),
	                unEditableDiffuseColor= new Color3f(.7f,0,0),
	                unEditableEmissiveColor= new Color3f(.2f,0,0),
	                selectedSpecularColor= new Color3f( .9f, .9f, 0 ),
	                selectedDiffuseColor= new Color3f( .6f, .6f, 0 ),
	                selectedEmissiveColor= new Color3f( .3f, .3f, 0 );
	private Blood blood;
	private Material fileMaterial;

	public FileObject(File file,Vector3f position, Bfm bfm ) {
		this.file= file;
		this.bfm= bfm;
		this.filePosition= position;
		setPickable(true);
		setCapability(BranchGroup.ALLOW_DETACH);
		setCapability(Group.ALLOW_CHILDREN_EXTEND);
		setCapability(TransformGroup.ENABLE_PICK_REPORTING);
		createShape(position);
		health= height;
	}

	public FileObject() {}

	/*public void animate( Vector3f v ) {
		Vector3f vector= new Vector3f();
		float dx,dz;			 
		moveT3D.get(vector);
		dx= v.x -vector.x ;
		dz=	v.z -vector.z;
		originalPositionT3D.get(vector);
		dx-= vector.x;
		dz-= vector.z;

		textRotateT3D.rotY(Math.atan2(dx,dz));
		textRotateTrans.setTransform(textRotateT3D);
	}*/
		
	public float calculateHeight() {
		float minHeight= 0.1f, maxHeight= 3, height;
		int size= (int)file.length();
		if(size > 1) {
			height= (float)Math.log(size)/20*maxHeight;
			if(height<minHeight) return minHeight;
			else if(height>maxHeight) return maxHeight;
			else return height;
		} else {
			return minHeight;
		}
	}

	public boolean checkXZIntersect(Vector3f p) {
		Vector3f v= new Vector3f();
		originalPositionT3D.get(v);
		float dx= v.x-p.x;
		float dz= v.z-p.z;
		if(Math.sqrt(dx*dx+dz*dz)<= radius) return true;
		else return false;
	} 

	public void bleed() {
		if( bleeding && bfm.getShowBlood() ) bleeding= blood.updateData( bloodOffset );
		else if( dead ) finished= true;
	}
	
	public void hit( float d ) {
		if( ( bfm.getPlayer().getMode().equals( "Remove" ) || bfm.getPlayer().getMode().equals( "Safe" ))) {
			if( d>0 && file.canWrite() ) {
				if( blood==null && bfm.getShowBlood() ) {
					BranchGroup b= new BranchGroup();
					Transform3D t3d= new Transform3D();
					t3d.setTranslation( new Vector3f(filePosition.x, -bloodOffset, filePosition.z));
					TransformGroup tg= new TransformGroup(t3d);
					tg.addChild(blood= new Blood(50));
					b.addChild(tg);
					addChild(b);
				}
				health-=d;
				bleeding= true;
				if( health<=0 ) {
					dead= true;
					moveT3D.setTranslation( new Vector3f( 0,-10,0 ));
					moveTrans.setTransform(moveT3D);
					if( !bfm.getSafeMode() ) {
						System.out.println("removing " + file.getName());
						bfm.getPlayer().addToStats( file.length() );
						file.delete();
						bfm.getPlayer().removeFromDamagedFiles( this, file.getPath() );
					}
				}
			}
			if( health<height ) {
				fileMaterial.setSpecularColor( new Color3f( .3f+((1-(health/height))*.7f), .3f+((1-(health/height))*.7f), 0 ));
				fileMaterial.setDiffuseColor( new Color3f( (1-(health/height))*.7f, ((1-(health/height))*.7f), 0 ));
				fileMaterial.setEmissiveColor( new Color3f( .2f+((1-(health/height))*.2f), .2f+((1-(health/height))*.2f), 0 ));
				bfm.getPlayer().addToDamagedFiles( this, file.getPath() );
			} else {
				resetColor();
			}
		} else if( bfm.getPlayer().getMode().equals( "Unselect" ) && file.canRead() && d>0 ) {
			selected= false;
			bfm.getPlayer().removeFromSelectedFiles( this, file.getPath() );
			resetColor();
		} else if(( bfm.getPlayer().getMode().equals( "Select" ) && file.canRead() && d>0 ) || selected ) {
			fileMaterial.setSpecularColor( selectedSpecularColor );
			fileMaterial.setDiffuseColor( selectedDiffuseColor );
			fileMaterial.setEmissiveColor( selectedEmissiveColor );
			selected= true;
			bfm.getPlayer().addToSelectedFiles( this, file.getPath() );
		} else {
			resetColor();
		}
	}

	public void unselect() {
		selected= false;
		resetColor();
	}

	public void resetColor() {
		if( bfm.getPlayer().getMode().equals( "Remove" ) ) {
			if( file.canWrite() ) {
				fileMaterial.setSpecularColor( editableSpecularColor );
				fileMaterial.setDiffuseColor( editableDiffuseColor );
				fileMaterial.setEmissiveColor( editableEmissiveColor );
			} else {
				fileMaterial.setSpecularColor( unEditableSpecularColor );
				fileMaterial.setDiffuseColor( unEditableDiffuseColor );
				fileMaterial.setEmissiveColor( unEditableEmissiveColor );
			}
		} else if( bfm.getPlayer().getMode().equals( "Select" ) || bfm.getPlayer().getMode().equals( "Unselect" )) {
			if( file.canRead() ) {
				fileMaterial.setSpecularColor( editableSpecularColor );
				fileMaterial.setDiffuseColor( editableDiffuseColor );
				fileMaterial.setEmissiveColor( editableEmissiveColor );
			} else {
				fileMaterial.setSpecularColor( unEditableSpecularColor );
				fileMaterial.setDiffuseColor( unEditableDiffuseColor );
				fileMaterial.setEmissiveColor( unEditableEmissiveColor );
			}
		}
	}

	TransformGroup createText(float height, Appearance textApp) {
		/*Transform3D t3d= new Transform3D();
		t3d.setTranslation(new Vector3f(0f,height/2+0.2f,0f));
		TransformGroup textMoveTrans= new TransformGroup(t3d);

		textRotateT3D= new Transform3D();
		textRotateTrans= new TransformGroup();
		textRotateTrans.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		textRotateTrans.setTransform(textRotateT3D);
		textMoveTrans.addChild(textRotateTrans);
		t3d= new Transform3D();
		t3d.setScale(.25f);
		TransformGroup objScale= new TransformGroup(t3d);
		textRotateTrans.addChild(objScale);
	
		String textString= file.getName();
		Font3D font3d= new Font3D( new Font("Arial", Font.PLAIN, 1),new FontExtrusion());
		Text3D text= new Text3D( font3d, textString);
		text.setAlignment( 0 );
		textShape= new Shape3D( text );
		textShape.setAppearance( textApp );

		objScale.addChild(textShape);
		return textMoveTrans;*/
		Transform3D t3d= new Transform3D();
		t3d.setTranslation(new Vector3f(0f,height/2+0.2f,0f));
		t3d.setScale(.25f);
		TransformGroup textRotateTrans= new TransformGroup(t3d);
	
		String textString= file.getName();
		Font3D font3d= new Font3D( new Font("Arial", Font.PLAIN, 1),new FontExtrusion());
		Text3D text= new Text3D( font3d, textString);
		text.setAlignment( 0 );

		OrientedShape3D orient = new OrientedShape3D();
		orient.addGeometry(text);
		orient.setAppearance(textApp);
		orient.setAlignmentAxis( 0.0f, 1.0f, 0.0f);
		
		textRotateTrans.addChild(orient);
		return textRotateTrans;
	}

	private void createShape(Vector3f positionVector) {
		height=calculateHeight();
   	
		originalPositionT3D= new Transform3D();
		originalPositionT3D.setTranslation(positionVector);
		originalPositionTrans= new TransformGroup(originalPositionT3D);
		addChild(originalPositionTrans);

		moveT3D= new Transform3D();
		moveT3D.setTranslation(new Vector3f(0f,height/2f,0f));
		moveTrans= new TransformGroup(moveT3D);
		moveTrans.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		originalPositionTrans.addChild(moveTrans);

		Appearance fileAppearance= new Appearance();
		fileAppearance.setCapability(Appearance.ALLOW_MATERIAL_WRITE);
		fileMaterial= new Material();
		fileMaterial.setCapability(Material.ALLOW_COMPONENT_WRITE);
		fileMaterial.setShininess(128);
		if(file.isHidden()) {
			TransparencyAttributes ta= new TransparencyAttributes(TransparencyAttributes.BLENDED,.4f);
			fileAppearance.setTransparencyAttributes(ta);
		}
		if(file.canWrite()) {
			fileMaterial.setDiffuseColor(editableDiffuseColor);
			fileMaterial.setSpecularColor(editableSpecularColor);
			fileMaterial.setEmissiveColor(editableEmissiveColor);
		} else {
			fileMaterial.setDiffuseColor(unEditableDiffuseColor);
			fileMaterial.setSpecularColor(unEditableSpecularColor);
			fileMaterial.setEmissiveColor(unEditableEmissiveColor);
		}
		fileAppearance.setMaterial( fileMaterial );
		//cylinder= new Cylinder(radius,height,fileAppearance);
		//moveTrans.addChild(cylinder);

		/* detect filetypes */
		String fileEnding;
		try {
		fileEnding = file.getName().substring(file.getName().lastIndexOf(".")+1,file.getName().length());
		} catch (StringIndexOutOfBoundsException e) {
			fileEnding = "";
		}

		fileEnding = fileEnding.toLowerCase();

		String[] documentEndings = {"java","c", "cpp","doc","abw","rtf"};
		String[] videoEndings = {"avi","mpg","mpeg","wmv"};
	String[] pictureEndings = {"png","bmp","gif","jpeg","jpg","psd"};
		
		if(stringContains(documentEndings,fileEnding))
			moveTrans.addChild(new FSDocument(fileAppearance,height,radius));
		else if(stringContains(videoEndings,fileEnding))
			moveTrans.addChild(new FSVideo(fileAppearance,height,radius*2));
		else if(stringContains(pictureEndings,fileEnding) && bfm.getShowThumbnails())
			moveTrans.addChild(new FSPicture(fileAppearance,height,radius*2,file));
		else
			moveTrans.addChild(new FSDefault(fileAppearance,height,radius));
		moveTrans.addChild(createText(height, fileAppearance));
	}

	boolean stringContains(String[] sa,String s) {
		for(int i=0;i<sa.length;i++) {
			if(sa[i].equals(s)) return true;
		}
		return false;
	}
	
	public void run( float f ) {}
	public void finish() {}
	public File getFile() { return file; }
	public float getRadius() { return radius; }
	public Vector3f getFilePosition() { return filePosition; }
	public float getHeight() { return height; }
	public boolean isSelected() { return selected; }
	public boolean isDamaged() { if( health<height ) return true; else return false; }
	public boolean isFinished() { return finished; }
}
