/*
    BFM, the Brutal File Manager. A 3D fps filemanager
    Copyright (C) 2003  Henrik Johansson and Daniel Forchheimer

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import com.sun.j3d.utils.geometry.*;
import javax.media.j3d.*;
import javax.vecmath.*;

public class GrapplingHook extends Weapon {
	private TransformGroup animateTrans, pos1, pos2, fireTrans;
	private BranchGroup fireGroup;
	private Appearance crosshairAppearance, weaponAppearance;
	private Vector3f move= new Vector3f( 0f, -.17f, -.5f ), firePosition= new Vector3f(), velocity;
	private float radius= .025f, length= .7f;
	private boolean hit= false;

	public GrapplingHook( Player player ) {
		super( player );
		animationFrames= 20;
		fireFrames= 1;
		fireDelay= .2f;
		repeatFire= true; repeatSecondaryFire= false;
		unlimitedFire= true;
		/* overrides addChild() in Weapon(), has to be called again */
		addChild( createShape() );
		addChild( getFireGroup() );
	}

	public void initializeAnimate() {
		Vector3f moveVector= new Vector3f( 0, move.y-(.02f*animationFrames)+.02f*currentAnimationFrame, move.z );
		t3d= new Transform3D();
		t3d.setTranslation( moveVector );
		animateTrans.setTransform(t3d);
		moveVector.add( new Vector3f( 0f, 0f, -length/2 ));
		t3d= new Transform3D();
		t3d.setTranslation( moveVector );
		fireTrans.setTransform(t3d);
	}

	public TransformGroup getCrosshair() {
		Box tinyBox= new Box( 2, 2, .002f, crosshairAppearance);
		TransformGroup crosshair= new TransformGroup();
		crosshair.addChild( tinyBox );
		return crosshair;
	}

	public TransformGroup createShape() {
		weaponAppearance= new Appearance();
		Material metal= new Material();
		metal.setDiffuseColor( new Color3f( .8f, .8f, .8f ));
		metal.setSpecularColor( new Color3f( 1f, 1f, 1f ));
		metal.setShininess( 128 );
		weaponAppearance.setMaterial( metal );

		animateTrans= new TransformGroup();
		animateTrans.setCapability(ALLOW_LOCAL_TO_VWORLD_READ);
		animateTrans.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
		animateTrans.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);

		Cylinder barrel= new Cylinder( radius, length, weaponAppearance );
		t3d= new Transform3D();
		t3d.rotX(-Math.PI/2);
		TransformGroup barrelGroup= new TransformGroup( t3d );
		barrelGroup.addChild( barrel );
		animateTrans.addChild( barrelGroup );

		pos1= new TransformGroup();
		pos1.setCapability(ALLOW_LOCAL_TO_VWORLD_READ);
		t3d= new Transform3D();
		t3d.setTranslation( new Vector3f( 0,0,-1f ));
		pos2= new TransformGroup(t3d);
		pos2.setCapability(ALLOW_LOCAL_TO_VWORLD_READ);
		pos2.addChild( pos1 );
		animateTrans.addChild( pos2 );

		return animateTrans;
	}

	private BranchGroup getFireGroup() {
		fireGroup= new BranchGroup();
		t3d= new Transform3D();
		t3d.setTranslation( new Vector3f( move.x, move.y, move.z-length/2 ));
		fireTrans= new TransformGroup(t3d);
		fireTrans.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
		fireTrans.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);

		float sphereRadius= radius*3;
		Sphere sphere= new Sphere( sphereRadius, weaponAppearance );
		fireTrans.addChild( sphere );

		for( int i=0; i<4; i++ ) {
			t3d= new Transform3D();
			t3d.rotX( i*Math.PI/4 );
			TransformGroup tg= new TransformGroup(t3d);
			int piggar= 8;
			float coneRadius= radius/2;
			float coneHeight= radius*2;
			for ( int n=0; n<piggar; n++ ) {
				double angle= 2*Math.PI/piggar*n;
				Cone cone= new Cone( coneRadius, coneHeight );
				t3d= new Transform3D();
				t3d.rotZ( angle );
				t3d.setTranslation( new Vector3f( -(sphereRadius+.4f*coneHeight)*(float)Math.sin(angle), (sphereRadius+.4f*coneHeight)*(float)Math.cos(angle), 0 ));
				TransformGroup singleConeGroup= new TransformGroup(t3d);
				singleConeGroup.addChild( cone );
				tg.addChild( singleConeGroup );
			}
			fireTrans.addChild(tg);
		}

		fireGroup.setCapability(ALLOW_DETACH);
		fireGroup.addChild( fireTrans );
		return fireGroup;
	}

	public void fireThisWeapon() {
		if( currentAnimationFrame==1 && !hit ) {
			t3d= new Transform3D();
			pos1.getLocalToVworld(t3d);
			firePosition= new Vector3f();
			t3d.get(firePosition);
			t3d= new Transform3D();
			pos2.getLocalToVworld(t3d);
			Vector3f v2= new Vector3f();
			t3d.get(v2);
			velocity= new Vector3f();
			velocity.sub(firePosition,v2);
			removeChild( fireGroup );
			room.addChild( fireGroup );
		}
		if( stillFiring && !hit ) {
			// ta bort room.addChild()
			// Players position -z varje gng
			// -firePosition i frhllande till den nuvarande positionen
			// forums.java.sun.com - distance vector camera roty
			fireFrames++;

			Vector3f position= new Vector3f( velocity );
			position.scale( currentAnimationFrame );
			position.add( firePosition);
			t3d= new Transform3D();
			t3d.setTranslation( position );
			fireTrans.setTransform( t3d );

			for( int i=0; i<room.numChildren(); i++ ) {
				try {
					if( ((FileObject)room.getChild(i)).checkXZIntersect(position) && position.y<((FileObject)room.getChild(i)).getHeight() ) {
						hit= true;
					}
				} catch( Exception e ) {}
			}
			if( position.y<0 || position.y>room.getHeight() || Math.abs( Math.sqrt( position.x*position.x+position.z*position.z ))>room.getRadius() ) hit= true;
		} else if( stillFiring && hit ) {
			//flytta spelaren
		}
		if( !stillFiring ) {
			room.removeChild( fireGroup );
			addChild( getFireGroup() );
			fireFrames= 1;
			hit= false;
		}
	}

	public void secondaryFireThisWeapon() {}

	public void endAnimate() {
		Vector3f moveVector= new Vector3f( 0, move.y-.02f*currentAnimationFrame, move.z );
		t3d= new Transform3D();
		t3d.setTranslation( moveVector );
		animateTrans.setTransform(t3d);
		moveVector.add( new Vector3f( 0f, 0f, -length/2 ));
		t3d= new Transform3D();
		t3d.setTranslation( moveVector );
		fireTrans.setTransform(t3d);
	}
}
