/*
    BFM, the Brutal File Manager. A 3D fps filemanager
    Copyright (C) 2003  Henrik Johansson and Daniel Forchheimer

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import com.sun.j3d.utils.geometry.*;
import java.awt.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.image.*;
import java.net.URL;

public class Hud extends BranchGroup {
	private Text2D currentDirText, aimingAtText, modeText, ammoText /*, chihuText */;
	private TransformGroup root, currentDir, mode, aimingAt, crosshair, progressbar, message, ammo /*, chihu */;
	private BranchGroup detach, progressbarDetach, scopeView;
	private float scale, scopeScale, scopeViewDisplacement= .01f, originalScale, zDisplacement= .11f, textScale= 0.024f, crosshairScale= .0001f;
	private double fovScale=1;
	private Dimension size;
	private int marginal=10, textMarginal= marginal, textSize= 18;
	private String textFace= "Arial", messageType="none";
	private Color3f hudColor= new Color3f( 1,1,0 );
	private Shape3D scopeViewShape;

	public Hud( Dimension size, double fov ) {
		/* stuff used for calculation of positions */
		this.size= size;
		scale= (((float)java.lang.Math.tan(fov/2.0)*zDisplacement)*2.0f)/(float)size.width;
		scopeScale= (((float)java.lang.Math.tan(fov/2.0)*(zDisplacement+scopeViewDisplacement))*2.0f)/(float)size.width;
		originalScale= scale;

		/* set hud capabilities */
		setCapability(ALLOW_DETACH);
		setCapability(ALLOW_CHILDREN_WRITE);
		setCapability(ALLOW_CHILDREN_EXTEND);

		/* displace everything -zDisplacement */
		Transform3D t3d= new Transform3D();
		t3d.setTranslation(new Vector3f(0,0,-zDisplacement));
		root= new TransformGroup(t3d);
		root.setCapability(TransformGroup.ALLOW_CHILDREN_READ);
		root.setCapability(TransformGroup.ALLOW_CHILDREN_EXTEND);
		root.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE);
		
		/* for everything but the progressbar */
		detach= new BranchGroup();
		detach.setCapability(ALLOW_DETACH);
		detach.setCapability(ALLOW_CHILDREN_WRITE);
		detach.setCapability(ALLOW_CHILDREN_EXTEND);

		/* for progressbar only, prevents text from flickering */
		progressbarDetach= new BranchGroup();
		progressbarDetach.setCapability(ALLOW_DETACH);
		progressbarDetach.setCapability(ALLOW_CHILDREN_WRITE);
		progressbarDetach.setCapability(ALLOW_CHILDREN_EXTEND);

		/* crosshair */
		crosshair= new TransformGroup();
		crosshair.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		detach.addChild(crosshair);

		/* progressbar */
		progressbar= new TransformGroup();
		progressbarDetach.addChild(progressbar);

		/* this probably needs an explanation :) */
		/* hihi!!!! */
		/*t3d= new Transform3D();
		t3d.setScale(textScale);
		t3d.setTranslation(new Vector3f(-.0435f,.025f,0));
		chihu= new TransformGroup(t3d);
		chihuText= new Text2D("uh4v3b3En1337h4Xx0rIzd!!!",new Color3f(1,1,0),"Arial",80,Font.PLAIN);
		chihu.addChild(chihuText); 
		detach.addChild(chihu);*/

		/* mode */
		t3d= new Transform3D();
		t3d.setScale(textScale);
		//t3d.setTranslation(new Vector3f((textMarginal-size.width/2)*scale,((textHeight+textMarginal)/2+textHeight/2-size.height/2)*scale,0));
		mode= new TransformGroup(t3d);
		mode.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		modeText= new Text2D("",hudColor,textFace,textSize,Font.PLAIN);
		mode.addChild(modeText); 
		detach.addChild(mode);

		/* aiming at text */
		t3d= new Transform3D();
		t3d.setScale(textScale);
		//t3d.setTranslation(new Vector3f((textMarginal-size.width/2)*scale,((textHeight+textMarginal)/2+textHeight/2-size.height/2)*scale,0));
		aimingAt= new TransformGroup(t3d);
		aimingAt.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		aimingAtText= new Text2D("",hudColor,textFace,textSize,Font.PLAIN);
		aimingAt.addChild(aimingAtText); 
		detach.addChild(aimingAt);

		/* current dir text */
		t3d= new Transform3D();
		t3d.setScale(textScale);
		//t3d.setTranslation(new Vector3f((textMarginal-size.width/2)*scale,(textHeight/2-size.height/2)*scale,0));
		currentDir= new TransformGroup(t3d);
		currentDir.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		currentDirText= new Text2D("",hudColor,textFace,textSize,Font.PLAIN);
		currentDir.addChild(currentDirText); 
		detach.addChild(currentDir);

		/* ammo */
		t3d= new Transform3D();
		t3d.setScale(textScale);
		//t3d.setTranslation(new Vector3f((textMarginal-size.width/2)*scale,(textHeight+textMarginal+textHeight/2-size.height/2)*scale,0));
		ammo= new TransformGroup(t3d);
		ammo.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		ammoText= new Text2D("",hudColor,textFace,2*textSize,Font.PLAIN);
		ammo.addChild(ammoText); 
		detach.addChild(ammo);

		/* help/stats message */
		message= new TransformGroup();
		message.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		detach.addChild(message);

		/* create scope view, but don't add it */
		t3d= new Transform3D();
		t3d.setTranslation( new Vector3f( 0f, 0f, -scopeViewDisplacement ));
		TransformGroup t= new TransformGroup(t3d);
		createScopeView();
		t.addChild(scopeViewShape);
		scopeView= new BranchGroup();
		scopeView.setCapability(BranchGroup.ALLOW_DETACH);
		scopeView.addChild(t);
		
		/* add everything */
		root.addChild(detach);
		root.addChild(progressbarDetach);
		addChild(root);
	}

	/* set/change current dir text */
	public void setCurrentDirText(String s) {
		detach.detach();
		currentDir.removeChild(currentDirText);
		currentDirText= new Text2D(s,hudColor,textFace,textSize,Font.PLAIN);
		currentDir.addChild(currentDirText);
		root.addChild(detach);
	}

	/* set/change mode text */
	public void setModeText( String s ) {
		detach.detach();
		mode.removeChild(modeText);
		modeText= new Text2D(s,hudColor,textFace,textSize,Font.PLAIN);
		mode.addChild(modeText);
		root.addChild(detach);
	}

	/* set/change aiming at */
	public void setAimingAtText( String s ) {
		detach.detach();
		aimingAt.removeChild(aimingAtText);
		aimingAtText= new Text2D(s,hudColor,textFace,textSize,Font.PLAIN);
		aimingAt.addChild(aimingAtText);
		root.addChild(detach);
	}

	/* set/change ammo text */
		public void setAmmoText( String s ) {
		detach.detach();
		ammo.removeChild(ammoText);
		ammoText= new Text2D(s,hudColor,textFace,2*textSize,Font.PLAIN);
		ammo.addChild(ammoText);
		root.addChild(detach);
	}

	/* set/change crosshair */
	public void setCrosshair( TransformGroup in ) {
		detach.detach();
		detach.removeChild(crosshair);
		Transform3D t3d= new Transform3D();
		t3d.setScale(crosshairScale);
		crosshair= in;
		crosshair.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		crosshair.setTransform(t3d);
		detach.addChild(crosshair);
		root.addChild(detach);
	}

	/* set/change progressbar */
	public void setProgressBar( int i, int frames ) {
		if( i> frames ) i= frames;
		float maxWidth= (size.width/2)*scale/2;
		float width= maxWidth*i/frames;
		progressbarDetach.detach();
		progressbarDetach.removeChild(progressbar);
		// use something else than a Box maybe, it doesn't feel right :)
		Box percentage= new Box( width, .0004f, .00001f, getAppearance(hudColor));
		if( i==frames ) percentage.setAppearance(getAppearance(new Color3f( 1,0,0 )));
		Transform3D t3d= new Transform3D();
		t3d.setTranslation(new Vector3f(width-maxWidth,-size.height/3*scale,0));
		progressbar= new TransformGroup(t3d);
		progressbar.addChild(percentage);
		progressbarDetach.addChild(progressbar);
		root.addChild(progressbarDetach);
	}

	/* hide progressbar */
	public void hideProgressBar() {
		progressbarDetach.detach();
		progressbarDetach.removeChild(progressbar);
		progressbar= new TransformGroup();
		progressbarDetach.addChild(progressbar);
		root.addChild(progressbarDetach);
	}

	/* create appearances */
	private Appearance getAppearance( Color3f color ) {
		Appearance appearance= new Appearance();
		Material material= new Material( color, color, color, color, 1 );
		appearance.setMaterial( material );
		TransparencyAttributes ta= new TransparencyAttributes(TransparencyAttributes.BLENDED,.6f);
		appearance.setTransparencyAttributes(ta);
		return appearance;
	}

	/* return crosshairAppearance to weapons */
	public Appearance getCrosshairAppearance() {
		Appearance crosshairAppearance= new Appearance();
		Material material= new Material( hudColor, hudColor, hudColor, hudColor, 1 );
		crosshairAppearance.setMaterial( material );
		return crosshairAppearance;
	}

	/* refresh HUD when the window is resized */
	public void refresh( Dimension size, double currentFov, double normalFov ) {
		this.size= size;
		fovScale= currentFov/normalFov;
		scale= (((float)java.lang.Math.tan(currentFov/2.0)*zDisplacement)*2.0f)/(float)size.width;
		scopeScale= (((float)java.lang.Math.tan(currentFov/2.0)*(zDisplacement+scopeViewDisplacement))*2.0f)/(float)size.width;
		textMarginal= (int)(fovScale*marginal*originalScale/scale);
		int calculatedTextSize= (int)(fovScale*textSize*originalScale/scale);
		calculateScopeView();

		/* move and resize text */
		Transform3D t3d= new Transform3D();
		t3d.setScale(fovScale*textScale);
		t3d.setTranslation( new Vector3f((textMarginal-size.width/2)*scale,(textMarginal-size.height/2)*scale,0 ));
		currentDir.setTransform(t3d);
		t3d= new Transform3D();
		t3d.setScale(fovScale*textScale);
		t3d.setTranslation( new Vector3f((textMarginal-size.width/2)*scale,(textMarginal-size.height/2+calculatedTextSize*2)*scale,0));
		ammo.setTransform(t3d);
		t3d= new Transform3D();
		t3d.setScale(fovScale*textScale);
		t3d.setTranslation( new Vector3f((textMarginal-size.width/2)*scale,(textMarginal-size.height/2+calculatedTextSize)*scale,0 ));
		mode.setTransform(t3d);
		t3d= new Transform3D();
		t3d.setScale(fovScale*textScale);
		t3d.setTranslation( new Vector3f((textMarginal-size.width/2)*scale,(size.height/2-textMarginal-calculatedTextSize)*scale,0 ));
		aimingAt.setTransform(t3d);

		t3d= new Transform3D();
		t3d.setScale(fovScale);
		if( !messageType.equals( "none" )) message.setTransform(t3d);
	}

	/* set message text and toggle on/off */
	public void toggleMessage( String[][] text, String newMessageType ) {
		if( messageType.equals( newMessageType )) {
			detach.detach();
			message.removeAllChildren();
			messageType= "none";
			root.addChild(detach);
		} else {
			messageType= newMessageType;
			setMessage( text );
		}
	}

	/* set message, used to update stats */
	public void setMessage( String[][] text ) {
		detach.detach();
		Transform3D t3d= new Transform3D();
		t3d.setScale(fovScale);
		message.removeAllChildren();
		message.setTransform(t3d);
		for( int n=0; n<2; n++ ) {
			for( int i=0; i<text.length; i++ ) {
				t3d= new Transform3D();
				t3d.setScale(textScale);
				t3d.setTranslation( new Vector3f( (1f/(float)fovScale)*((-size.width/4+(n*size.width/3))*scale),(1f/(float)fovScale)*(-textSize*scale/2+(textSize+textMarginal)/2*(text.length/2-i)*scale),0 ));
				TransformGroup textTG= new TransformGroup(t3d);
				Text2D messageText= new Text2D( text[i][n], hudColor, textFace, textSize, Font.PLAIN );
				textTG.addChild(messageText);
				message.addChild(textTG);
			}
		}
		root.addChild(detach);
	}

	/* show scope view when zooming */
	public void showScopeView() {
		detach.detach();
		detach.addChild( scopeView );
		root.addChild( detach );
	}

	/* hide scope view when unzooming */
	public void hideScopeView() {
		scopeView.detach();
	}

	/* create scope view for zooming */
	private void createScopeView() {
		QuadArray scopeViewCenter= new QuadArray(4, QuadArray.COORDINATES | QuadArray.TEXTURE_COORDINATE_2);
		scopeViewCenter.setCapability(GeometryArray.ALLOW_COORDINATE_WRITE);
		QuadArray scopeViewSides= new QuadArray(8, QuadArray.COORDINATES | QuadArray.NORMALS);
		scopeViewSides.setCapability(GeometryArray.ALLOW_COORDINATE_WRITE);
		
		Appearance texApp= new Appearance();
		TransparencyAttributes tra = new TransparencyAttributes();
		tra.setTransparencyMode( TransparencyAttributes.BLENDED );
		texApp.setTransparencyAttributes( tra );
		URL url=this.getClass().getResource("data/images/scope.png");
    TextureLoader loader = new TextureLoader(url, null);
    Texture2D texture = (Texture2D) loader.getTexture();
    texApp.setTexture(texture);

		TexCoord2f[]q = new TexCoord2f[4];
		q[0]= new TexCoord2f(0.0f, 1.0f);
		q[1]= new TexCoord2f(0.0f, 0.0f);
		q[2]= new TexCoord2f(1.0f, 0.0f);
		q[3]= new TexCoord2f(1.0f, 1.0f);
		scopeViewCenter.setTextureCoordinates(0,0,q);

		scopeViewShape= new Shape3D(scopeViewCenter, texApp);
		scopeViewShape.setCapability(Shape3D.ALLOW_GEOMETRY_READ);
		scopeViewShape.addGeometry(scopeViewSides);
		calculateScopeView();
	}
	
	/* calculate positions for scope view */
	private void calculateScopeView() {
		float width= 0;
		if( size.width>size.height ) {
			/* add 10px to make sure it covers the Canvas3D */
			width= (size.height+10)/2*scopeScale;
			Point3f[] p= new Point3f[8];
			p[0]= new Point3f(-scopeScale*size.width/2, -width, -0.0f);
			p[1]= new Point3f(-width, -width, -0.0f);
			p[2]= new Point3f(-width, width, -0.0f);
			p[3]= new Point3f(-scopeScale*size.width/2, width, -0.0f);
			p[4]= new Point3f( width, -width, -0.0f);
			p[5]= new Point3f(scopeScale*size.width/2, -width, -0.0f);
			p[6]= new Point3f(scopeScale*size.width/2, width, -0.0f);
			p[7]= new Point3f(width, width, -0.0f);
			((QuadArray)(scopeViewShape.getGeometry(1))).setCoordinates(0,p);
		} else {
			/* add 10px to make sure it covers the Canvas3D */
			width=(size.width+10)/2*scopeScale;
			Point3f[] p= new Point3f[8];
			p[0]= new Point3f(-width, -scopeScale*size.height/2, -0.0f);
			p[1]= new Point3f(width, -scopeScale*size.height/2, -0.0f);
			p[2]= new Point3f(width, -width, -0.0f);
			p[3]= new Point3f(-width, -width, -0.0f);
			p[4]= new Point3f(width, width, -0.0f);
			p[5]= new Point3f(width, scopeScale*size.height/2, -0.0f);
			p[6]= new Point3f(-width, scopeScale*size.height/2, -0.0f);
			p[7]= new Point3f(-width, width, -0.0f);
			((QuadArray)(scopeViewShape.getGeometry(1))).setCoordinates(0,p);
		}
		Point3f[] p= new Point3f[4];
		p[0]= new Point3f(-width, width, -0.0f);
		p[1]= new Point3f(-width, -width, -0.0f);
		p[2]= new Point3f( width, -width, -0.0f);
		p[3]= new Point3f( width, width, -0.0f);
		((QuadArray)(scopeViewShape.getGeometry(0))).setCoordinates(0,p);
	}

	/* return currently displayed message type */
	public String getMessageType() { return messageType; }
}
