/*
    BFM, the Brutal File Manager. A 3D fps filemanager
    Copyright (C) 2003  Henrik Johansson and Daniel Forchheimer

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
import javax.swing.plaf.*;
import java.awt.*;
import java.awt.event.*;
import java.io.File;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import javax.swing.border.Border;

public class OpenDialog extends JPanel implements ActionListener {
	private final String OKBUTTONTEXT= "GO!", CANCELBUTTONTEXT= "Cancel";
	private JTextField textField;
	private String startPath, title;
	private Bfm bfm;

	public OpenDialog( Bfm bfm, String title, String startPath ) {
		this.bfm= bfm;
		this.title= title;
		this.startPath= startPath;
		setLayout( new BoxLayout( this, BoxLayout.Y_AXIS ));
		setBorder( BorderFactory.createMatteBorder( 0, 5, 0, 5, Color.BLACK ));
		Border lineBorder= BorderFactory.createLineBorder( Color.YELLOW, 1 );

		java.net.URL imageURL= getClass().getResource( "data/images/foldericon.gif" );
		ImageIcon icon= new ImageIcon( imageURL );
		UIDefaults defaults = UIManager.getDefaults();
		defaults.put("Button.foreground", new ColorUIResource( Color.YELLOW ));
		defaults.put("Button.background", new ColorUIResource( Color.BLACK ));
		defaults.put("Panel.foreground", new ColorUIResource( Color.YELLOW ));
		defaults.put("Panel.background", new ColorUIResource( Color.BLACK ));
		defaults.put("TextField.background", new ColorUIResource( Color.BLACK ));
		defaults.put("TextField.foreground", new ColorUIResource( Color.YELLOW ));
		defaults.put("TextField.selectionBackground", new ColorUIResource( new Color( 190, 190, 0 )));
		defaults.put("TextField.selectionForeground", new ColorUIResource( Color.BLACK ));
		defaults.put("Tree.closedIcon", new IconUIResource( icon ));
		defaults.put("Tree.leafIcon", new IconUIResource( icon ));
		defaults.put("Tree.openIcon", new IconUIResource( icon ));
		defaults.put("Tree.background", new ColorUIResource( Color.BLACK ));
		defaults.put("Tree.foreground", new ColorUIResource( Color.YELLOW ));
		defaults.put("Tree.selectionBackground", new ColorUIResource( Color.BLACK ));
		defaults.put("Tree.selectionBorderColor", new ColorUIResource( Color.BLACK ));
		defaults.put("Tree.selectionForeground", new ColorUIResource( Color.WHITE));
		defaults.put("Tree.textBackground", new ColorUIResource( Color.BLACK ));
		defaults.put("Tree.textForeground", new ColorUIResource( Color.YELLOW ));
		
		addComponentListener( new ComponentAdapter() {
			public void componentShown( ComponentEvent ce ) {
				textField.requestFocusInWindow();
				textField.selectAll();
			}
		});

		/* TextField */
		JPanel textFieldPanel= new JPanel();
		textFieldPanel.setBorder( BorderFactory.createEmptyBorder( 0, 0, 5, 0 ));
		textFieldPanel.setLayout( new BorderLayout() );
		textField= new JTextField( startPath );
		textField.setBorder( lineBorder );
		textFieldPanel.setMaximumSize( new Dimension( 2000, 40 ));
		textField.addActionListener( this );
		textFieldPanel.add( textField );
		add( textFieldPanel, BorderLayout.PAGE_START );
		
		/* FileSystemTree (JTree) in JScrollPane */
		FileSystemTree tree;
		File[] roots= File.listRoots();
		if( roots.length>1 ) {
			DefaultMutableTreeNode treeRoot= new DefaultMutableTreeNode( "My computer" );
			for (int k=0; k < roots.length; k++) {
				FileNode node= new FileNode( roots[k] );
				treeRoot.add( node );
				node.explore();
			}
			tree= new FileSystemTree( textField, new DefaultTreeModel( treeRoot ));
		} else {
			FileNode node= new FileNode( roots[0] );
			node.explore();
			tree= new FileSystemTree( textField, new DefaultTreeModel( node ));
		}
		JScrollPane scrollPane= new JScrollPane( tree );
		scrollPane.setBorder( lineBorder );
		add( scrollPane, BorderLayout.CENTER );

		/* cancel and go buttons */
		Border buttonBorder= BorderFactory.createCompoundBorder( BorderFactory.createLineBorder( Color.YELLOW, 1 ), BorderFactory.createEmptyBorder( 3, 3, 3, 3 ));
		JPanel buttonPanel= new JPanel();
		JButton go= new JButton( OKBUTTONTEXT );
		go.setBorder( buttonBorder );
		go.addActionListener( this );
		buttonPanel.add( go );
		JButton cancel= new JButton( CANCELBUTTONTEXT );
		cancel.setBorder( buttonBorder );
		cancel.addActionListener( this );
		buttonPanel.add( cancel );
		add( buttonPanel, BorderLayout.PAGE_END );
	}

	public void actionPerformed( ActionEvent ae ) {
		if( ae.getActionCommand().equals( CANCELBUTTONTEXT )) clearAndHide();
		else testString( textField.getText() );
	}

	private boolean testString( String path ) {
		boolean successful= true;
		if( path.equals( startPath )) clearAndHide();
		else if( new File( path ).isDirectory() && new File( path ).canRead() ) {
			clearAndHide();
			bfm.setNewRoom( path );
		} else {
			successful= false;
			String errorMessage="could not be loaded";
			if ( ( new File( path ).isFile() )) errorMessage= "is a file, not a directory";
			else if( !( new File( path ).isDirectory() )) errorMessage= "is not a valid path";
			else if( !( new File( path ).canRead() )) errorMessage= "is unreadable";
			textField.selectAll();
			JOptionPane.showMessageDialog(this,"Sorry, \""+path+"\" "+errorMessage+".\nPlease try again",title+" - Error, invalid path!",JOptionPane.ERROR_MESSAGE);
			path= null;
			textField.requestFocusInWindow();
		}
		return successful;
	}
	
	public void clearAndHide() {
		((CardLayout)bfm.getLayout()).show( bfm, "canvasCard" );
		textField.setText(null);
		setVisible(false);
		bfm.hideCursor();
		bfm.resetMouse();
		bfm.resume();
	}

	public void refresh( String path ) {
		textField.setText( path );
		textField.selectAll();
	}
}

class FileSystemTree extends JTree {
	public FileSystemTree( final JTextField textField, DefaultTreeModel dtm ) {
		super( dtm );
		setLayout( new BorderLayout() );	

		addTreeSelectionListener( new TreeSelectionListener() {
			public void valueChanged(TreeSelectionEvent event) {
				DefaultMutableTreeNode node= (DefaultMutableTreeNode)event.getPath().getLastPathComponent();
				Object obj= node.getUserObject();
				FileNode apa= (FileNode)node;
				textField.setText( apa.getFile().toString() );
			}
		} );
		addTreeExpansionListener( new TreeExpansionListener() {
			public void treeCollapsed( TreeExpansionEvent e ) {}
			public void treeExpanded( TreeExpansionEvent e ) {
				TreePath path= e.getPath();
				FileNode node= (FileNode)path.getLastPathComponent();

				if ( !node.isExplored() ) {
					DefaultTreeModel model= (DefaultTreeModel)getModel();
					node.explore();
					model.nodeStructureChanged( node );
				}
			}
		} );
	}
}

class FileNode extends DefaultMutableTreeNode {
	private boolean explored= false;

	public FileNode( File file ) { setUserObject( file ); }
	public boolean getAllowsChildren() { return isDirectory(); }
	public boolean isLeaf() { return!isDirectory(); }
	public File getFile() { return ( File )getUserObject(); }
	public boolean isExplored() { return explored; }
	public boolean isDirectory() { return getFile().isDirectory(); }

	public String toString() {
		File file= ( File )getUserObject();
		String filename= file.toString();
		int index= filename.lastIndexOf( File.separator );

		return ( index != -1 && index != filename.length() - 1 ) ?
			filename.substring( index + 1 ) : filename;
	}

	public void explore() {
		if ( !isDirectory() ) return;
			if ( !isExplored() && getFile().canRead() ) {
			File file= getFile();
			File[] children= file.listFiles();
			
			for ( int i= 0; i < children.length; ++i ) if( children[i].isDirectory() ) add( new FileNode( children[i] ) );
			explored= true;
		}
	}
}
