/*
    BFM, the Brutal File Manager. A 3D fps filemanager
    Copyright (C) 2003  Henrik Johansson and Daniel Forchheimer

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import javax.media.j3d.*;
import javax.vecmath.*;
import java.io.*;

public class PhysicsHandler implements Runnable {
	private Bfm bfm;
	private Player player;
	private Room room;
	private int frame;
	private float TIMEINTERVAL=0.02f /* in seconds */, GRAVITY= 6f, marginal=.2f, fileRadius, groundLevel=0, playerHeight;

	public PhysicsHandler( Player player, Bfm bfm ) {
		this.bfm= bfm;
		this.player= player;
		room= bfm.getRoom();
		fileRadius= room.getFileRadius()+marginal;
		playerHeight= player.height;
		startThread();
	}

	public void startThread() {
		Thread t= new Thread( this);
		t.start();
	}

	public void run() {
		//long lastTimeMillis = System.currentTimeMillis();
		
		while( !bfm.getPause() ) {
			frame++;
			room= bfm.getRoom();
			Vector3f v= player.getVelocity();
			
			/* gunga */
			if( !player.jumping && ( v.x!=0 || v.z!=0 )) player.gunga();

			/* apply gravitation (always) */
			v.y-= GRAVITY*TIMEINTERVAL;

			Vector3f p= player.calculatePosition();

			/* check collition between player and files */
			groundLevel= 0;
			for(int i=0;i<room.numChildren();i++) try { ((FileObject)room.getChild(i)).bleed(); } catch( Exception e ) {}
			for(int i=0;i<room.numChildren();i++) {
				try {
					Vector3f filePos=((FileObject)room.getChild(i)).getFilePosition();
					//((FileObject)room.getChild(i)).animate(player.getPositionVector());
					float deltaX=p.x-filePos.x;
					float deltaZ=p.z-filePos.z;
					if( Math.sqrt( (double)(deltaX*deltaX+deltaZ*deltaZ) )<fileRadius ) {
						if( p.y>playerHeight || (player.crouching && p.y>playerHeight/2) ) {
							groundLevel= ((FileObject)room.getChild(i)).getHeight();
						} else {
							double fi=Math.atan2( deltaX, deltaZ );
							if(fi<0) fi+=Math.PI+Math.PI;
							if( player.crouching ) p= new Vector3f(new Point3f( (float)(Math.sin(fi)*fileRadius+filePos.x), playerHeight/2, (float)(Math.cos(fi)*fileRadius+filePos.z) ));
							else p= new Vector3f(new Point3f( (float)(Math.sin(fi)*fileRadius+filePos.x), p.y, (float)(Math.cos(fi)*fileRadius+filePos.z) ));
						}
						break;
					}
				} catch( Exception e ) {}
			}

			/* check collisions with walls */
			double distanceFromCenter= Math.abs( Math.sqrt( p.x*p.x+p.z*p.z ));
			if( distanceFromCenter+1.2f*marginal>room.getRadius() ) {
				player.standingCloseToWall= true;
				if( distanceFromCenter+marginal>room.getRadius() || player.using ) {
					float angle= (float)Math.atan2(p.x,p.z);
					if(angle<0) angle+=Math.PI+Math.PI;
					String s= room.getClosestWall(angle);
					if( s!="" && new File( s ).canRead() && ( bfm.getAutoChangeRooms() || player.using )) {
						String previousDir= room.getPath();
						room= bfm.setNewRoom( s );
						p= room.getNewPositions( player, previousDir );
					} else {
						p= new Vector3f(new Point3f( (float)Math.sin(angle)*(room.getRadius()-marginal), p.y, (float)Math.cos(angle)*(room.getRadius()-marginal) ));
					}
				}
			} else {
				player.standingCloseToWall= false;
			}
			player.using= false;

			/* prevent player from falling through the ground, 
				 if standing on a file groundLevel= height of the file */
			if( p.y-playerHeight<groundLevel ) {
				if(player.crouching) p.y=playerHeight/2+groundLevel;
				else p.y=playerHeight+groundLevel;
				v.y=0;
				player.jumping=false;
			}

			/* prevent player from jumping through the ceiling */
			if( p.y>room.getHeight()-marginal ) p.y=room.getHeight()-marginal;

			/* move Rockets, add delay to DroppedDynamite, remove any of them if exploded */
			for( int i=0; i<room.numChildren(); i++ ) {
				try {
					RoomObject roomObject= (RoomObject)(room.getChild(i));
					if( roomObject.isFinished() ) room.removeChild(i);
					else roomObject.run( TIMEINTERVAL );
				} catch( Exception e ) {}
			}

			/* move player, fire, initialize, end and change weapons */
			player.move(p);
			if( (!player.getWeapon().isFiring() || player.getWeapon().getTimer()>0) && player.getNewWeapon()!=-1 ) {
				player.getWeapon().finishFiring();
				player.changeWeapon( player.getNewWeapon() );
				player.setNewWeapon( -1 );
			}
			if( !player.getWeapon().isFiring() ) {
				if( player.endWeapon && player.getWeapon().end() ) { player.endWeapon= false; player.setWeapon(); }
				if( player.initializeWeapon && player.getWeapon().initialize() ) player.initializeWeapon= false;
			}
			if( !player.initializeWeapon && !player.endWeapon ) {
				if( player.fire || player.getWeapon().isFiring() ) player.getWeapon().fire( bfm.getRoom(), TIMEINTERVAL, player.fire );
				if( player.secondaryFire ) player.getWeapon().secondaryFire( bfm.getRoom() );
			}
			if( (frame*TIMEINTERVAL*1000)>=400 ) { frame=0; player.getWeapon().updateAimingAt( room ); }

			/* pause if player is dead */
			if( !player.isAlive() ) {
				player.getHud().setModeText( "G A M E   O V E R" );
				player.getHud().setCurrentDirText( "press Fire to continue" );
				break;
			}
			
			/* original method for timing */
			try { Thread.sleep( (int)(TIMEINTERVAL*1000) ); } catch( InterruptedException ie ) {}

			/* new method for more accurat timing, doesn't work very well though*/
			/* 
			long dt = (long)(TIMEINTERVAL*1000)-System.currentTimeMillis() + lastTimeMillis;
			//System.out.println(dt);
			if(dt>0)
				try { Thread.sleep(  dt ); } catch( InterruptedException ie ) {}
			lastTimeMillis = System.currentTimeMillis();
			*/
		}
	}
}
