/*
    BFM, the Brutal File Manager. A 3D fps filemanager
    Copyright (C) 2003  Henrik Johansson and Daniel Forchheimer

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import com.sun.j3d.utils.universe.*; 
import javax.media.j3d.*;
import javax.vecmath.*;
import java.util.*;
import java.awt.*;

public class Player extends BranchGroup {
	public float height= 1.8f, speedJump= 0.9f, speed= .2f, walkSpeed= .2f, runSpeed= .4f, gungFi= 0;
	public boolean initializeWeapon= true, endWeapon= false, alive= true, standingCloseToWall= false, toggleRun= false, running= false, using= false, crouching= false, jumping= false, fire= false, secondaryFire= false;
	private long deletedBytes= 0;
	private int weaponIndex= 0, deletedFiles= 0, newWeapon= -1;
	private double fov, viewAngleXZ= 0, viewAngleY= 0; 
	private Hud hud;
	private Weapon weapon;
	private BranchGroup rightHand;
	private TransformGroup rightHandTrans, viewTransform;
	private Transform3D moveT3D,rotT3D;
	private Vector3f positionVector= new Vector3f(0,height,0), velocity= new Vector3f( 0, 0, 0 );
	private String mode="Remove";
	private ArrayList selectedFiles= new ArrayList(), damagedFiles= new ArrayList(), selectedPaths= new ArrayList(), damagedPaths= new ArrayList();
	private Spear spear;
	private Shotgun shotgun;
	private SniperRifle sniperRifle;
	private Dynamite dynamite;
	private RocketLauncher rocketLauncher;
	//private GrapplingHook grapplingHook;
	private SimpleUniverse universe;

	public Player( SimpleUniverse universe, double fov ) {
		this.universe= universe;
		this.fov= fov;
		universe.addBranchGraph( this );
		viewTransform= universe.getViewingPlatform().getViewPlatformTransform();

		rotT3D= new Transform3D();
		moveT3D= new Transform3D();
		moveT3D.setTranslation(positionVector);

		/* create a new hud */
		hud= new Hud( universe.getCanvas().getSize(), fov );
		viewTransform.setTransform(moveT3D);
		viewTransform.addChild(hud);
		hud.refresh( universe.getCanvas().getSize(), fov, fov );

		/* create all weapons */
		spear= new Spear( this );
		shotgun= new Shotgun( this );
		sniperRifle= new SniperRifle( this );
		dynamite= new Dynamite( this );
		rocketLauncher= new RocketLauncher( this );
		//grapplingHook= new GrapplingHook( this );

		/* set weapon Player starts with */
		weapon= spear;

		/* add weapon */
		rightHand= new BranchGroup();
		rightHand.setCapability(BranchGroup.ALLOW_DETACH);
		rightHandTrans= new TransformGroup();
		rightHandTrans.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		rightHandTrans.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
		rightHandTrans.addChild(weapon);

		rightHand.addChild(rightHandTrans);
		viewTransform.addChild(rightHand);
	}

	public void gunga() {
		gungFi+=0.3;
		Transform3D t3d= new Transform3D();
		t3d.setTranslation(new Vector3f(0f,(float)(0.01*Math.sin(gungFi)),0f));
		rightHandTrans.setTransform(t3d);
	}

	public Vector3f getVelocity() { return velocity; }
		
	public void setVelocityX( float f ) { velocity.x= f; }
	public void setVelocityZ( float f ) { velocity.z= f; }

	public void lookUp(double fiXZ) {
		viewAngleXZ+= fiXZ;
		if( viewAngleXZ>Math.PI/4 ) viewAngleXZ=Math.PI/4;
		else if( viewAngleXZ<-Math.PI/4 ) viewAngleXZ=-Math.PI/4;
		moveT3D.rotX(viewAngleXZ);
	}

	public void turnRight(double fiY) {
		viewAngleY+= fiY;
		rotT3D.rotY(viewAngleY);
		rotT3D.mul(moveT3D);
	}

	public double getViewAngleXZ() { return viewAngleXZ; }
	public double getViewAngleY() { return viewAngleY; }

	public void setYAxisRotation( double d ) {
		turnRight(d-viewAngleY);
	}

	public void move(Vector3f moveVector) {
		positionVector= moveVector;
		rotT3D.setTranslation(positionVector);
		viewTransform.setTransform(rotT3D);
	}
	
	public Vector3f calculatePosition() {
		Vector3f v= new Vector3f();
		v.x= -velocity.z*(float)Math.sin(viewAngleY)+velocity.x*(float)Math.cos(viewAngleY);
		v.z= -velocity.z*(float)Math.cos(viewAngleY)-velocity.x*(float)Math.sin(viewAngleY);
		v.y= velocity.y;
		v.add(positionVector);
		return v;
	}

	/* methods dealing with selected files */
	public void addToSelectedFiles( FileObject f, String s ) {
		if( !selectedFiles.contains( f )) {
			selectedFiles.add( f );
			selectedPaths.add( s );
		}
	}
	public void removeFromSelectedFiles( FileObject f, String s ) {
		if( selectedFiles.contains( f )) {
			selectedFiles.remove( selectedFiles.indexOf( f ));
			selectedPaths.remove( selectedPaths.indexOf( s ));
		}
	}
	public ArrayList getSelectedFiles() { return selectedFiles; }
	public ArrayList getSelectedPaths() { return selectedPaths; }
	public void resetSelectedFiles( Room room ) {
		selectedFiles.clear();
		selectedPaths.clear();
		for( int i=0; i<room.numChildren(); i++ ) {
			try {
				((FileObject)(room.getChild(i))).unselect();
			} catch( Exception e ) {}
		}
	}

	/* methods dealing with damaged files */
	public void addToDamagedFiles( FileObject f, String s ) {
		if( !damagedFiles.contains( f )) {
			damagedFiles.add( f );
			damagedPaths.add( s );
		}
	}
	public void removeFromDamagedFiles( FileObject f, String s ) {
		if( damagedFiles.contains( f )) {
			damagedFiles.remove( damagedFiles.indexOf( f ));
			damagedPaths.remove( damagedPaths.indexOf( s ));
		}
	}
	public ArrayList getDamagedFiles() { return damagedFiles; }
	public ArrayList getDamagedPaths() { return damagedPaths; }

	/* a _lot_ of misc methods :) */
	public void setNewWeapon( int i ) { newWeapon= i; }
	public int getNewWeapon() { return newWeapon; };
	public Vector3f getPositionVector() { return positionVector; }
	public Hud getHud() { return hud; }
	public Weapon getWeapon() { return weapon; }
	public void remove() { viewTransform.removeChild(rightHand); viewTransform.removeChild(hud); }
	public boolean isAlive() { return alive; }
	public void setAlive( boolean b ) { alive= b; }
	public String getMode() { return mode; }
	public void setMode( String s ) { mode= s; hud.setModeText( "Mode: "+s+ " mode" ); }

	/* methods for changing weapons */
	public void changeWeapon( int i ) { if( weaponIndex!=i ) { weaponIndex= i; endWeapon= true; } }
	public void nextWeapon( int numberOfWeapons ) { weaponIndex= (weaponIndex+1)%numberOfWeapons; endWeapon= true; }
	public void prevWeapon( int numberOfWeapons ) { weaponIndex= (weaponIndex-1+numberOfWeapons)%numberOfWeapons; endWeapon= true; }
	public void setWeapon() {
		rightHand.detach();
		rightHandTrans.removeChild(weapon);
		     if( weaponIndex == 0 ) weapon= spear;
		else if( weaponIndex == 1 ) weapon= shotgun;
		else if( weaponIndex == 2 ) weapon= sniperRifle;
		else if( weaponIndex == 3 ) weapon= dynamite;
		else if( weaponIndex == 4 ) weapon= rocketLauncher;
		//else if( weaponIndex == 5 ) weapon= grapplingHook;
		rightHandTrans.addChild(weapon);
		viewTransform.addChild(rightHand);
		initializeWeapon= true;
	}

	public void jump() {
		if( !jumping && !crouching ) {
			velocity.y= speedJump;
			jumping= true;
		}
	}

	public void toggleCrouch() {
		if( !jumping ) {
			if( !crouching ) {
				crouching=true;
				if( running ) changeSpeed();
				if( toggleRun ) toggleRun();
				velocity.scale( .5f );
			} else {
				crouching=false;
				velocity.scale( 2 );
			}
		}
	}
 
	public void toggleRun() {
		if( !crouching ) {
			if( !toggleRun ) {
				toggleRun=true;
				velocity.scale( 2 );
				speed=runSpeed;
			} else {
				toggleRun=false;
				velocity.scale( .5f );
				speed=walkSpeed;
			}
		}
	}
	
	public void changeSpeed() {
		if( !toggleRun && !crouching ) {
			if( !running ) {
				running=true;
				velocity.scale( 2 );
				speed=runSpeed;
			} else {
				running=false;
				velocity.scale( .5f );
				speed=walkSpeed;
			}
		}
	}

	public void addToStats( long bytes ) {
		deletedFiles++;
		deletedBytes+= bytes;
		if( hud.getMessageType().equals( "stats" )) hud.setMessage( getStats() );
	}

	public String[][] getStats() {
		String[][] stats= { { "Deleted files", String.valueOf(deletedFiles) }, 
		                    { "Deleted bytes", String.valueOf(deletedBytes) } };
		return stats;
	}

	public void zoom( float scale ) {
		hud.refresh( universe.getCanvas().getSize(), fov*scale, fov );
		universe.getViewer().getView().setFieldOfView( fov*scale );
	}
}
