/*
    BFM, the Brutal File Manager. A 3D fps filemanager
    Copyright (C) 2003  Henrik Johansson and Daniel Forchheimer

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import com.sun.j3d.utils.geometry.*;
import javax.media.j3d.*;
import javax.vecmath.*;

public class Rocket extends RoomObject {
	private float expand= .15f, damage=.1f;
	private int explotionFrames= 30, index;
	private Room room;
	private Vector3f velocity, position;
	private TransformGroup animateTrans= new TransformGroup();

	public Rocket( float barrelLength, Transform3D trans, Room room, Vector3f position, Vector3f velocity ) {
		this.room= room;
		this.position= position;
		this.velocity= velocity;
		animateTrans.setTransform( trans );
		float radius=0.075f;

		Appearance weaponAppearance= new Appearance();
		Material material= new Material();
		material.setDiffuseColor( new Color3f( .8f, .8f, .8f ));
		material.setSpecularColor( new Color3f( 1f, 1f, 1f ));
		weaponAppearance.setMaterial( material );

		Cylinder cylinder= new Cylinder( radius, barrelLength/3, weaponAppearance );
		Transform3D t3d= new Transform3D();
		t3d.rotX( -Math.PI/2 );
		t3d.setTranslation( new Vector3f( 0,0,barrelLength/6+barrelLength/10 ));
		TransformGroup cylinderGroup= new TransformGroup( t3d );
		cylinderGroup.addChild( cylinder );

		float coneLength= barrelLength/5;
		Cone cone= new Cone(radius, coneLength, weaponAppearance );
		t3d= new Transform3D();
		t3d.rotX( -Math.PI/2 );
		TransformGroup coneGroup= new TransformGroup( t3d );
		coneGroup.addChild( cone );

		animateTrans.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
		animateTrans.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		animateTrans.addChild(cylinderGroup);
		animateTrans.addChild(coneGroup);
		addChild( animateTrans );
	}

	public void run( float f ) {
		/* move Rocket, call getTransform to preserve rotation */
		Transform3D t3d= new Transform3D();
		animateTrans.getTransform(t3d);
		t3d.setTranslation(position);
		animateTrans.setTransform(t3d);

		/* kill() if hit */
		for( int i=0; i<room.numChildren(); i++ ) {
			try {
				if( ((FileObject)room.getChild(i)).checkXZIntersect(position) && position.y<((FileObject)room.getChild(i)).getHeight() ) {
					explode();
					break;
				}
			} catch( Exception e ) {}
		}
		if( position.y<0 || position.y>room.getHeight() || Math.abs( Math.sqrt( position.x*position.x+position.z*position.z ))>room.getRadius() ) explode();
		position.add( velocity );
	}

	private void explode() {
		Explotion explotion= new Explotion( explotionFrames, expand, room, position );
		room.addChild( explotion );
		finished= true;
	}

	public void finish() { while( !finished ) run( 0f );
	}
}
