/*
    BFM, the Brutal File Manager. A 3D fps filemanager
    Copyright (C) 2003  Henrik Johansson and Daniel Forchheimer

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import com.sun.j3d.utils.geometry.*;
import javax.media.j3d.*;
import javax.vecmath.*;

public class RocketLauncher extends Weapon {
	private TransformGroup trans, pos1, pos2;
	private float expand= .15f, damage=.1f, length;
	private Vector3f move= new Vector3f( .3f, -.05f, -.5f );
	private int explotionFrames= 40;

	public RocketLauncher( Player player ) {
		super( player );
		animationFrames= 20;
		fireFrames= 1;
		fireDelay= 1f;
		repeatFire= true; repeatSecondaryFire= false;
		/*ammo=2; */unlimitedSecondaryFire= true; unlimitedFire= true;
	}

	public void initializeAnimate() {
		t3d= new Transform3D();
		t3d.rotX( Math.PI/2-((float)currentAnimationFrame/(float)animationFrames)*Math.PI/2 );
		t3d.setTranslation( move );
		trans.setTransform(t3d);
	}

	public TransformGroup getCrosshair() {
		Box tinyBox= new Box( 2, 2, .002f, player.getHud().getCrosshairAppearance());		TransformGroup crosshair= new TransformGroup();
		crosshair.addChild( tinyBox );
		return crosshair;
	}

	TransformGroup createShape() {
		float radius=0.08f, length=1;
		this.length= length;

		Appearance weaponAppearance= new Appearance();
		Material material= new Material();
		material.setEmissiveColor( new Color3f( 0, .2f, 0 ));
		material.setSpecularColor( new Color3f( 0, .4f, 0 ) );
		material.setDiffuseColor( new Color3f( 0, .3f, 0 ));
		weaponAppearance.setMaterial( material );

		Cylinder barrel= new Cylinder( radius, length, weaponAppearance );
		t3d= new Transform3D();
		t3d.rotX( -Math.PI/2 );
		TransformGroup barrelGroup= new TransformGroup( t3d );
		barrelGroup.addChild( barrel );

		pos1= new TransformGroup();
		pos1.setCapability(ALLOW_LOCAL_TO_VWORLD_READ);
		t3d= new Transform3D();
		t3d.setTranslation( new Vector3f( 0,0,-1f ));
		pos2= new TransformGroup(t3d);
		pos2.setCapability(ALLOW_LOCAL_TO_VWORLD_READ);
		pos2.addChild( pos1 );

		trans= new TransformGroup(t3d);
		trans.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
		trans.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		trans.addChild( barrelGroup );
		trans.addChild( pos2 );
		return trans;
	}

	public void fireThisWeapon() {
		/* calculate position of Rocket */
		t3d= new Transform3D();
		pos1.getLocalToVworld(t3d);
		Vector3f v1= new Vector3f();
		t3d.get(v1);
		t3d= new Transform3D();
		pos2.getLocalToVworld(t3d);
		Vector3f v2= new Vector3f();
		t3d.get(v2);
		Vector3f velocity= new Vector3f();
		velocity.sub(v1,v2);

		/* create Rocket, move and rotate it */
		t3d= new Transform3D();
		t3d.rotX( player.getViewAngleXZ() );
		Transform3D t3d2= new Transform3D();
		t3d2.rotY( player.getViewAngleY() );
		t3d2.mul(t3d);
		t3d2.setTranslation(v1);
		Rocket rocket= new Rocket( length, t3d2, room, v1, velocity );

		/* add Rocket to Room and start animation */
		room.addChild( rocket );
	}

	public void secondaryFireThisWeapon() {}

	public void endAnimate() {
		t3d= new Transform3D();
		t3d.rotX( ((float)currentAnimationFrame/(float)animationFrames)*Math.PI/2 );
		t3d.setTranslation( move );
		trans.setTransform(t3d);
	}
}
