/*
    BFM, the Brutal File Manager. A 3D fps filemanager
    Copyright (C) 2003  Henrik Johansson and Daniel Forchheimer

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import javax.media.j3d.*;
import javax.vecmath.*;
import java.io.*;
import java.util.*;
import com.sun.j3d.utils.image.*;
import com.sun.j3d.utils.picking.*;
import java.net.URL;


public class Room extends BranchGroup {
	private static int MINNUMBEROFWALLS=8;
	private static float MINRADIUS= 10;
	private Wall[] walls;
	private ArrayList directoryList= new ArrayList(), fileList= new ArrayList();
	private float radius;
	private int height= 6, wallNumber;
	private Bfm bfm;
	private Pattern pattern;
	private float wallAngles[];
	private String path, floorImage="data/images/floor.jpg";
	
	public Room( String path, Bfm bfm ) {
		this.path= path;
		this.bfm= bfm;
		setCapability(BranchGroup.ALLOW_DETACH);
		setCapability(BranchGroup.ALLOW_CHILDREN_READ);
		setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
		setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);

		pattern= new PatternCircle( new FileObject().getRadius() );
		File dir= new File( path );
		File[] files= dir.listFiles();
		java.util.Arrays.sort( files );

		try { directoryList.add( new File( dir.getParent() )); } catch( Exception e ) {}

		for( int i=0; i<files.length; i++ ) {
			if( files[i].isDirectory() ) directoryList.add(files[i]);
			else fileList.add(files[i]);
		}
		while(directoryList.size()<MINNUMBEROFWALLS) directoryList.add(null);
		radius= directoryList.size();
		if(radius<MINRADIUS) radius=MINRADIUS;
		pattern.setMinRadius(directoryList.size(),fileList.size(),radius);
		if(radius<pattern.getMinRadius()) radius=pattern.getMinRadius();
		double fi= (2*Math.PI/directoryList.size());
		/* *3 == fulfix */
		double length= 3*radius*Math.tan( fi/2 );
 	
		/* *3 == filfix */
		Shape3D floor= createPlane(radius*3);
		Shape3D ceiling= createPlane(radius*3);
 	
		walls= new Wall[directoryList.size()];
		wallAngles= new float[directoryList.size()];

	/* create walls */
		for( int i=0; i<directoryList.size(); i++ ) {
			try {
				walls[i]= new Wall((File)directoryList.get(i), dir, (int)length, height );
			} catch(NullPointerException e) {
				walls[i]= new Wall((int)length, height );
			}
			TransformGroup rotateWall= new TransformGroup();
			Transform3D t3d= new Transform3D();
			t3d.rotY( i*fi );
			wallAngles[i]= (float)(i*fi);
			rotateWall.setTransform( t3d );
			
			TransformGroup moveWall= new TransformGroup();
			t3d= new Transform3D();
			Vector3f justMoveTheDamnWall= new Vector3f( 0, (float)(height/2f), radius );
			t3d.setTranslation( justMoveTheDamnWall );
			moveWall.setTransform( t3d );
			
			moveWall.addChild( walls[i] );
			rotateWall.addChild( moveWall );
			addChild( rotateWall ); 		
		}

		/* create files */
		for(int i=0;i<fileList.size();i++) {
			FileObject fileObject;
			if( bfm.getPlayer().getDamagedPaths().contains( ((File)fileList.get(i)).getPath() )) {
				fileObject= (FileObject)bfm.getPlayer().getDamagedFiles().get( bfm.getPlayer().getDamagedPaths().indexOf( ((File)fileList.get(i)).getPath() ));
				fileObject.detach();
			} else if( bfm.getPlayer().getSelectedPaths().contains( ((File)fileList.get(i)).getPath() )) {
				fileObject= (FileObject)bfm.getPlayer().getSelectedFiles().get( bfm.getPlayer().getSelectedPaths().indexOf( ((File)fileList.get(i)).getPath() ));
				fileObject.detach();
			} else {
				fileObject= new FileObject((File)fileList.get(i),pattern.getPosition(i),bfm );
			}
			addChild( fileObject );
		}

		PointLight lampa= new PointLight();
		lampa.setInfluencingBounds( new BoundingSphere( new Point3d( 0, 0, 0 ), 100 ));
		addChild( lampa );
	
		TransformGroup tgceiling= new TransformGroup();
		Transform3D t3d= new Transform3D();
		t3d.rotX(Math.PI);
		t3d.setTranslation( new Vector3f( 0, height, 0 ));
		tgceiling.setTransform( t3d );

		addChild( floor );
		tgceiling.addChild( ceiling );
		addChild( tgceiling );

		enablePicking(this);
		
		compile();
	}

	/* from http://www.java3d.org/selection.html */
	public void enablePicking(Node node) {
		node.setPickable(true);
		node.setCapability(Node.ENABLE_PICK_REPORTING);
		try {
			Group group = (Group) node;
			for (Enumeration e = group.getAllChildren(); e.hasMoreElements();) {
				enablePicking((Node)e.nextElement());
			}
		} catch(ClassCastException e) { /* if not a group node, there are no children so ignore exception */ }
		try {
			Shape3D shape= (Shape3D)node;
			PickTool.setCapabilities(node, PickTool.INTERSECT_FULL);
			for (Enumeration e = shape.getAllGeometries(); e.hasMoreElements();) {
				try {
					Geometry g = (Geometry)e.nextElement();
					g.setCapability(g.ALLOW_INTERSECT);
				} catch (NullPointerException ne) {}
			}
		} catch(ClassCastException e) { /* not a Shape3D node ignore exception */ }
		  catch( RestrictedAccessException rae ) {}
	}

	private Shape3D createPlane(float length) {
		GeometryArray g;
		Shape3D s;
		g= new QuadArray(4,QuadArray.COORDINATES|QuadArray.TEXTURE_COORDINATE_2);
		TexCoord2f[] t=new TexCoord2f[4];
		int wrapDistance=(int)length>>2;
		t[0]=new TexCoord2f(0,wrapDistance);
		t[1]=new TexCoord2f(wrapDistance,wrapDistance);
		t[2]=new TexCoord2f(wrapDistance,0);
		t[3]=new TexCoord2f(0,0);
		g.setTextureCoordinates(0,0,t);

		float l= length/2;
		Point3f[] tt=new Point3f[4];
		tt[0]=new Point3f(-l,0,l);
		tt[1]=new Point3f(l,0,l);
		tt[2]=new Point3f(l,0,-l);
		tt[3]=new Point3f(-l,0,-l);
		g.setCoordinates(0,tt);

		Appearance floorAppearance= new Appearance();
		/* the url makes it possible to package the program in a jar-file */
		URL url= Room.class.getResource(floorImage);
		Texture texture= new TextureLoader( url, null).getTexture();
		texture.setBoundaryModeS(Texture.WRAP);
		texture.setBoundaryModeT(Texture.WRAP);
		floorAppearance.setTexture( texture );
	
		s= new Shape3D(g,floorAppearance);
		return s; 
	}

	public String getClosestWall(float angle) {
		int bestWall=0;
		float bestAngle=7;
		for(int i=0;i<wallAngles.length;i++) {
			if(Math.abs(wallAngles[i]-angle)<bestAngle) {
				bestWall= i;
				bestAngle= Math.abs(wallAngles[i]-angle);
				if( i==0 && 2*Math.PI-angle<bestAngle ) {
					bestWall=i;
					bestAngle=2*(float)Math.PI-angle;
				}
			}
		}
		return walls[bestWall].getPath();
	}

	public Vector3f getNewPositions( Player player, String previousDir ) {
		for( int i=0; i<directoryList.size(); i++ ) {
			if( walls[i].getPath().equals( previousDir )) wallNumber=i;
		}
		float displaceX=(float)(radius*Math.sin(wallNumber*2*Math.PI/directoryList.size() ));
		float displaceZ=(float)(radius*Math.cos(wallNumber*2*Math.PI/directoryList.size() ));
		double rotateCamera=( (wallNumber*2*Math.PI/directoryList.size() ));
		player.setYAxisRotation( rotateCamera );
		Vector3f newPlayerPosition= new Vector3f(new Point3f( displaceX, player.height, displaceZ ));
		return newPlayerPosition;
	}

	/* copy selected files */
	public void copyFiles( ArrayList temp ) {
		ArrayList filesToUnselect= new ArrayList();
		BranchGroup bg= new BranchGroup();
		for( int i=0; i<temp.size(); i++ ) {
			File newFile= new File( path+File.separator+((FileObject)temp.get(i)).getFile().getName().toString() );
			if( !((FileObject)temp.get(i)).getFile().getPath().equals( newFile.getPath() )) {
				try	{
					FileInputStream inputStream = new FileInputStream(((FileObject)temp.get(i)).getFile());
					FileOutputStream outputStream = new FileOutputStream(newFile);
					int size = inputStream.available();
					byte[] chunks = new byte[size];
					inputStream.read(chunks, 0, size);
					outputStream.write(chunks, 0, chunks.length);
					inputStream.close();
					outputStream.close();
				} catch( IOException e ) {
					System.out.println( "an error occured when copying "+ ((FileObject)temp.get(i)).getFile().getPath().toString()+" to "+newFile.getPath().toString() );
				}
				FileObject newFileObject= new FileObject(newFile,((FileObject)temp.get(i)).getFilePosition(),bfm );
				bg.addChild( newFileObject );
				fileList.add( "apa" );
				filesToUnselect.add( (FileObject)temp.get(i) );
			} else {
				((FileObject)temp.get(i)).unselect();
			}
		}
		addChild(bg);
		for( int i=0; i<filesToUnselect.size(); i++ ) {
			bfm.getPlayer().removeFromSelectedFiles( (FileObject)filesToUnselect.get(i), ((FileObject)filesToUnselect.get(i)).getFile().getPath() );
		}
		filesToUnselect.clear();
	}

	/* move selected files */
	public void moveFiles( ArrayList temp ) {
		ArrayList filesToUnselect= new ArrayList();
		BranchGroup bg= new BranchGroup();
		for( int i=0; i<temp.size(); i++ ) {
			File newFile= new File( path+File.separator+((FileObject)temp.get(i)).getFile().getName().toString() );
			if( !((FileObject)temp.get(i)).getFile().getPath().equals( newFile.getPath() ) && ((FileObject)temp.get(i)).getFile().renameTo( newFile )) {
				FileObject newFileObject= new FileObject(newFile,((FileObject)temp.get(i)).getFilePosition(),bfm );
				bg.addChild( newFileObject );
				fileList.add( "apa" );
				filesToUnselect.add( (FileObject)temp.get(i) );
			} else if( ((FileObject)temp.get(i)).getFile().getPath().equals( newFile.getPath() )) {
				((FileObject)temp.get(i)).unselect();
			}
		}
		addChild(bg);
		for( int i=0; i<filesToUnselect.size(); i++ ) {
			bfm.getPlayer().removeFromSelectedFiles( (FileObject)filesToUnselect.get(i), ((FileObject)filesToUnselect.get(i)).getFile().getPath() );
		}
		filesToUnselect.clear();
	}

	/* various methods */
	public Bfm getBfm() { return bfm; }
	public Wall[] getWalls() { return walls; }
	public float getRadius() { return radius; }
	public ArrayList getFileList() { return fileList; }
	public String getPath() { return path; }
	public int getHeight() { return height; }
	public float getFileRadius() { return new FileObject().getRadius(); }

}
