/*
    BFM, the Brutal File Manager. A 3D fps filemanager
    Copyright (C) 2003  Henrik Johansson and Daniel Forchheimer

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import com.sun.j3d.utils.geometry.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.picking.*;

public class Shotgun extends Weapon {
	private TransformGroup edgeOfPipe,startOfPipe, animateTrans;
	private Transform3D animateT3D= new Transform3D();	
	private Appearance crosshairAppearance;
	private float damage=.1f, spread=(float)Math.PI/16;

	public Shotgun( Player player ) {
		super( player );
		crosshairAppearance= player.getHud().getCrosshairAppearance();
		animationFrames= 15;
		fireFrames= 12;
		fireDelay= .4f;
		repeatFire= true; repeatSecondaryFire= false;
		/*ammo= 25; */unlimitedSecondaryFire= true; unlimitedFire= true;
	}

	public void initializeAnimate() {
		t3d= new Transform3D();
		animateTrans.getTransform(t3d);
		t3d.setTranslation( new Vector3f( 0, (-.02f*animationFrames)+.02f*currentAnimationFrame, 0 ));
		animateTrans.setTransform(t3d);
	}

	public TransformGroup getCrosshair() {
		TransformGroup crosshair= new TransformGroup();
		float displacement= 25, crosshairThin= 1, crosshairThick= 5;
		t3d= new Transform3D();
		t3d.setTranslation( new Vector3f( 0, displacement, 0 ) );
		Box north= new Box( crosshairThin, crosshairThick, crosshairThin, crosshairAppearance);
		TransformGroup northtg= new TransformGroup(t3d);
		northtg.addChild( north );
		t3d= new Transform3D();
		t3d.setTranslation( new Vector3f( displacement, 0, 0 ) );
		Box east= new Box( crosshairThick, crosshairThin, crosshairThin, crosshairAppearance);
		TransformGroup easttg= new TransformGroup(t3d);
		easttg.addChild( east );
		t3d= new Transform3D();
		t3d.setTranslation( new Vector3f( 0, -displacement, 0 ) );
		Box south= new Box( crosshairThin, crosshairThick, crosshairThin, crosshairAppearance);
		TransformGroup southtg= new TransformGroup(t3d);
		southtg.addChild( south );
		t3d= new Transform3D();
		t3d.setTranslation( new Vector3f( -displacement, 0, 0 ) );
		Box west= new Box( crosshairThick, crosshairThin, crosshairThin, crosshairAppearance);
		TransformGroup westtg= new TransformGroup(t3d);
		westtg.addChild( west );
		crosshair.addChild( northtg );
		crosshair.addChild( easttg );
		crosshair.addChild( southtg );
		crosshair.addChild( westtg );
		return crosshair;
	}
		
	TransformGroup createShape() {
		float radius=0.017f, length=.7f, moveX=radius, moveY=-.17f, moveZ=-.5f;

		Appearance weaponAppearance= new Appearance();
		Material metal= new Material();
		metal.setDiffuseColor( new Color3f( .8f, .8f, .8f ));
		metal.setSpecularColor( new Color3f( 1f, 1f, 1f ));
		metal.setShininess( 128 );
		weaponAppearance.setMaterial( metal );

		Cylinder leftBarrel= new Cylinder( radius, length, weaponAppearance );
		t3d= new Transform3D();
		t3d.rotX(-Math.PI/2);
		t3d.setTranslation( new Vector3f( -moveX, moveY, moveZ) );
		TransformGroup leftBarrelGroup= new TransformGroup( t3d );
		leftBarrelGroup.addChild( leftBarrel );

		Cylinder rightBarrel= new Cylinder( radius, length, weaponAppearance );
		t3d= new Transform3D();
		t3d.rotX(-Math.PI/2);
		t3d.setTranslation( new Vector3f( moveX, moveY, moveZ) );
		TransformGroup rightBarrelGroup= new TransformGroup( t3d );
		rightBarrelGroup.addChild( rightBarrel );

		Box box= new Box( radius, length/2, .01f, weaponAppearance );
		t3d= new Transform3D();
		t3d.rotX(-Math.PI/2);
		t3d.setTranslation( new Vector3f( 0, moveY, moveZ) );
		TransformGroup boxGroup= new TransformGroup( t3d );
		boxGroup.addChild( box );
 
		t3d= new Transform3D();
		t3d.setTranslation(new Vector3f(0,0f,-1f));
		edgeOfPipe= new TransformGroup(t3d);
		edgeOfPipe.setCapability(ALLOW_LOCAL_TO_VWORLD_READ);

		startOfPipe=new TransformGroup();
		startOfPipe.setCapability(ALLOW_LOCAL_TO_VWORLD_READ);
		
		animateTrans= new TransformGroup();
		animateTrans.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
		animateTrans.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		animateTrans.addChild(leftBarrelGroup);
		animateTrans.addChild(rightBarrelGroup);
		animateTrans.addChild(boxGroup);
		edgeOfPipe.addChild(startOfPipe);	 
		animateTrans.addChild(edgeOfPipe);
		return animateTrans;
	}

	public void fireThisWeapon() {
		/* kill immediately */
		if( currentAnimationFrame==1 ) {
			Transform3D pos1= new Transform3D();
			edgeOfPipe.getLocalToVworld(pos1);
			Vector3d v1= new Vector3d();
			pos1.get(v1);

			Transform3D pos0= new Transform3D();
			startOfPipe.getLocalToVworld(pos0);
			Vector3d v0= new Vector3d();
			pos0.get(v0);

			Vector3d dir= new Vector3d();
			dir.sub(v0,v1);

			PickTool pt= new PickTool(room);
			Point3d p= new Point3d(new Point3f(v0));

			for( int i=0; i<30; i++ ) {
				pt.setShapeRay(p,new Vector3d( dir.x+(Math.random()-.5f)*spread, dir.y+(Math.random()-.5f)*spread, dir.z+(Math.random()-.5f)*spread ));
				PickResult[] pr= pt.pickAllSorted();
				for(int n=0;n<pr.length;n++) {
					try {
						if(pr[n].getNode(PickResult.BRANCH_GROUP).getClass().getName().equals("FileObject")) {
							FileObject fo= (FileObject)pr[n].getNode(PickResult.BRANCH_GROUP);
							fo.hit( damage );
						}
					} catch(NullPointerException np) {} 
				}
			}
		}
		/* animate firing */		
		animateT3D.rotX(0.2f*Math.sin(Math.PI*((currentAnimationFrame+1)%fireFrames)/fireFrames));
		animateTrans.setTransform(animateT3D);
	}

	public void secondaryFireThisWeapon() {}

	public void endAnimate() {
		t3d= new Transform3D();
		animateTrans.getTransform(t3d);
		t3d.setTranslation( new Vector3f( 0, -.02f*currentAnimationFrame, 0 ));
		animateTrans.setTransform(t3d);
	}
}
