/*
    BFM, the Brutal File Manager. A 3D fps filemanager
    Copyright (C) 2003  Henrik Johansson and Daniel Forchheimer

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import com.sun.j3d.utils.geometry.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.picking.*;
import java.util.*;

public class SniperRifle extends Weapon {
	private TransformGroup edgeOfPipe, startOfPipe, animateTrans;
	private Transform3D animateT3D= new Transform3D(), t3d;	
	private float damage=1.7f, crosshairThin= 1, crosshairThick= 5;
	private boolean zooming= false;

	public SniperRifle( Player player ) {
		super( player );
		animationFrames= 20;
		fireFrames= 12;
		fireDelay= 1f;
		unlimitedSecondaryFire= true;
		repeatFire= true; repeatSecondaryFire= false;
		/*ammo= 5;*/unlimitedFire= true;
	}

	public void initializeAnimate() {
		t3d= new Transform3D();
		t3d.rotX( Math.PI/3-((float)currentAnimationFrame/(float)animationFrames)*Math.PI/3 );
		t3d.setTranslation( new Vector3f( 0, -1+((float)currentAnimationFrame/(float)animationFrames), 1.5f-1.5f*((float)currentAnimationFrame/(float)animationFrames) ));
		animateTrans.setTransform(t3d);
	}

	public TransformGroup getCrosshair() {
		TransformGroup crosshair= new TransformGroup();
		Box vertical= new Box( crosshairThin, crosshairThick, crosshairThin, player.getHud().getCrosshairAppearance());
		Box horizontal= new Box( crosshairThick, crosshairThin, crosshairThin, player.getHud().getCrosshairAppearance());
		crosshair.addChild( vertical );
		crosshair.addChild( horizontal );
		player.getHud().setCrosshair( crosshair );
		t3d= new Transform3D();
		animateTrans.setTransform(t3d);
		return crosshair;
	}

	TransformGroup createShape() {
		float thinRadius=0.025f, length=1.5f, moveX=.1f, moveY=-.15f, moveZ=-.5f, scopeYPosition= 3*thinRadius, coneHeight= length/21, thickRadius= 1.5f*thinRadius, mainScopeLength= length/7;

		Appearance weaponAppearance= new Appearance();
		Material metal= new Material();
		metal.setDiffuseColor( new Color3f( .8f, .8f, .8f ));
		metal.setSpecularColor( new Color3f( 1f, 1f, 1f ));
		metal.setShininess( 128 );
		weaponAppearance.setMaterial( metal );

		Cylinder barrel= new Cylinder( thinRadius, length, weaponAppearance );
		t3d= new Transform3D();
		t3d.rotX(-Math.PI/2);
		t3d.setTranslation( new Vector3f( moveX, moveY, moveZ ));
		TransformGroup barrelGroup= new TransformGroup(t3d);
		barrelGroup.addChild( barrel );

		Cylinder scopeFront= new Cylinder( thickRadius, coneHeight, weaponAppearance );
		t3d= new Transform3D();
		t3d.rotX(-Math.PI/2);
		t3d.setTranslation( new Vector3f( moveX, moveY+scopeYPosition, moveZ+mainScopeLength/2+coneHeight ));
		TransformGroup scopeFrontGroup= new TransformGroup(t3d);
		scopeFrontGroup.addChild( scopeFront );

		Cone frontCone= new Cone( thickRadius, coneHeight, weaponAppearance );
		t3d= new Transform3D();
		t3d.rotX(-Math.PI/2);
		t3d.setTranslation( new Vector3f( moveX, moveY+scopeYPosition, moveZ+mainScopeLength/2 ));
		TransformGroup frontConeGroup= new TransformGroup(t3d);
		frontConeGroup.addChild( frontCone );

		Cylinder mainScope= new Cylinder( thinRadius, mainScopeLength, weaponAppearance );
		t3d= new Transform3D();
		t3d.rotX(-Math.PI/2);
		t3d.setTranslation( new Vector3f( moveX, moveY+scopeYPosition, moveZ ));
		TransformGroup mainScopeGroup= new TransformGroup(t3d);
		mainScopeGroup.addChild( mainScope );

		Cone backCone= new Cone( thickRadius, coneHeight, weaponAppearance );
		t3d= new Transform3D();
		t3d.rotX(Math.PI/2);
		t3d.setTranslation( new Vector3f( moveX, moveY+scopeYPosition, moveZ-mainScopeLength/2 ));
		TransformGroup backConeGroup= new TransformGroup(t3d);
		backConeGroup.addChild( backCone );

		Cylinder scopeBack= new Cylinder( thickRadius, coneHeight, weaponAppearance );
		t3d= new Transform3D();
		t3d.rotX(-Math.PI/2);
		t3d.setTranslation( new Vector3f( moveX, moveY+scopeYPosition, moveZ-mainScopeLength/2-coneHeight ));
		TransformGroup scopeBackGroup= new TransformGroup(t3d);
		scopeBackGroup.addChild( scopeBack );

		Box box= new Box( thinRadius/3, thinRadius, coneHeight, weaponAppearance );
		t3d= new Transform3D();
		t3d.setTranslation( new Vector3f( moveX, moveY+thinRadius, moveZ ));
		TransformGroup boxGroup= new TransformGroup(t3d);
		boxGroup.addChild( box );

		t3d= new Transform3D();
		t3d.setTranslation(new Vector3f(0,0f,-1f));
		edgeOfPipe= new TransformGroup(t3d);
		edgeOfPipe.setCapability(ALLOW_LOCAL_TO_VWORLD_READ);

		startOfPipe=new TransformGroup();
		startOfPipe.setCapability(ALLOW_LOCAL_TO_VWORLD_READ);
		
		animateTrans= new TransformGroup();
		animateTrans.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		animateTrans.addChild( barrelGroup );
		animateTrans.addChild( scopeFrontGroup );
		animateTrans.addChild( frontConeGroup );
		animateTrans.addChild( mainScopeGroup );
		animateTrans.addChild( backConeGroup );
		animateTrans.addChild( scopeBackGroup );
		animateTrans.addChild( boxGroup );
	 
		edgeOfPipe.addChild(startOfPipe);	 
		animateTrans.addChild(edgeOfPipe); 
		return animateTrans;
	}

	void fireThisWeapon() {
		/* kill immediately */
		if( currentAnimationFrame==1 ) {
			Transform3D pos1= new Transform3D();
			edgeOfPipe.getLocalToVworld(pos1);
			Vector3d v1= new Vector3d();
			pos1.get(v1);
		
			Transform3D pos0= new Transform3D();
			startOfPipe.getLocalToVworld(pos0);
			Vector3d v0= new Vector3d();
			pos0.get(v0);

			Vector3d dir= new Vector3d();
			dir.sub(v0,v1);
		
			PickTool pt= new PickTool(room);
			Point3d p= new Point3d(new Point3f(v0));
		
			pt.setShapeRay(p,dir);
			pt.setMode(PickTool.GEOMETRY);
			PickResult[] pr= pt.pickAllSorted();
			for(int n=0;n<pr.length;n++) {
				try {
					if(pr[n].getNode(PickResult.BRANCH_GROUP).getClass().getName().equals("FileObject")) {
						FileObject fo= (FileObject)pr[n].getNode(PickResult.BRANCH_GROUP);
						fo.hit( damage );
						break;
					}
				} catch(NullPointerException np) {} 
			}
		}
		/* animate */
		animateT3D.setTranslation( new Vector3f( 0, 0, (float)(.6f*Math.sin(Math.PI*((currentAnimationFrame+1)%fireFrames)/fireFrames) )));
		animateTrans.setTransform(animateT3D);
	}
	
	void secondaryFireThisWeapon() {
		if( !zooming ) {
			zooming= true;
			player.getHud().setCrosshair( new TransformGroup());
			player.zoom( 1f/4f );
			player.getHud().showScopeView();
		} else {
			zooming= false;
			player.getHud().setCrosshair( getCrosshair() );
			player.getHud().hideScopeView();
			player.zoom( 1f );
		}
	}

	public void endAnimate() {
		if( currentAnimationFrame==1) {
			player.zoom( 1f );
			player.getHud().hideScopeView();
		}
		t3d= new Transform3D();
		t3d.rotX( ((float)currentAnimationFrame/(float)animationFrames)*Math.PI/3 );
		t3d.setTranslation( new Vector3f( 0, -((float)currentAnimationFrame/(float)animationFrames), 1.5f*((float)currentAnimationFrame/(float)animationFrames) ));
		animateTrans.setTransform(t3d);
	}
}
