/*
    BFM, the Brutal File Manager. A 3D fps filemanager
    Copyright (C) 2003  Henrik Johansson and Daniel Forchheimer

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import com.sun.j3d.utils.geometry.*;
import javax.media.j3d.*;
import javax.vecmath.*;

public class Spear extends Weapon {
	private TransformGroup edgeOfSpear, animateTrans;
	private float damage= .4f;

	public Spear( Player player ) {
		super( player );
		animationFrames= 15;
		fireFrames= 10;
		fireDelay= .35f;
		repeatFire= true; repeatSecondaryFire= true;
		unlimitedFire= true;
	}

	public void initializeAnimate() {
		t3d= new Transform3D();
		animateTrans.getTransform(t3d);
		t3d.setTranslation( new Vector3f( 0, (-.08f*animationFrames)+.08f*currentAnimationFrame, 0 ));
		animateTrans.setTransform(t3d);
	}

	public TransformGroup getCrosshair() {
		return new TransformGroup();
	}

	TransformGroup createShape() {
		float moveX= 0.15f, moveY= 0, moveZ= -1.5f, coneRadius= 0.1f, coneLength= 1f;
		t3d= new Transform3D();
		animateTrans= new TransformGroup(t3d);
		animateTrans.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
		animateTrans.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
 
		/* skapa spjutet */
		t3d= new Transform3D();
		t3d.rotX(-Math.PI/2);
		t3d.setTranslation(new Vector3f(moveX,moveY,moveZ));
		TransformGroup trans= new TransformGroup(t3d);
		t3d= new Transform3D();
		t3d.rotZ(0.2f);
		TransformGroup rotyTrans= new TransformGroup(t3d);
		rotyTrans.addChild(animateTrans);

		/* skapa spjutspetsen */
		Cone c= new Cone(coneRadius,coneLength);
		Material weaponMaterial= new Material( new Color3f(0.5f,0.5f,0.7f), new Color3f(0.5f,0.5f,0.7f), new Color3f(0.5f,0.5f,0.7f), new Color3f(0.9f,0.9f,1f), 1 );
		Appearance weaponAppearance= new Appearance();
		weaponAppearance.setMaterial(weaponMaterial);
		c.setAppearance(weaponAppearance);

		animateTrans.addChild(c);

		/* skapa handtag till spjutet */
		t3d= new Transform3D();
		t3d.setTranslation(new Vector3f(0,-coneLength/2-.6f,0));
		TransformGroup moveCylinder= new TransformGroup(t3d);
		Cylinder cyl= new Cylinder(coneRadius/2,2f);
		weaponMaterial= new Material();
		weaponMaterial.setDiffuseColor(new Color3f(.4f,.3f,.07f));
		weaponMaterial.setSpecularColor(new Color3f(.7f,.2f,.2f));
		weaponMaterial.setShininess(1);
		weaponMaterial.setEmissiveColor(new Color3f(0.4f,0.3f,0.07f));
		weaponAppearance= new Appearance();
		weaponAppearance.setMaterial(weaponMaterial);
		cyl.setAppearance(weaponAppearance);
		moveCylinder.addChild(cyl);
		animateTrans.addChild(moveCylinder);

		trans.addChild(rotyTrans);

		/* definera den del av spjutet som ddar */
		t3d= new Transform3D();
		t3d.setTranslation(new Vector3f(0f,coneLength/2,0));
		edgeOfSpear= new TransformGroup(t3d);
		edgeOfSpear.setCapability(ALLOW_LOCAL_TO_VWORLD_READ);
		animateTrans.addChild(edgeOfSpear);

		return trans;
	}

	public void fireThisWeapon() {
		/* animate */
		t3d= new Transform3D();
		t3d.setTranslation( new Vector3f( 0f, (float)(Math.sin(Math.PI*((currentAnimationFrame+1)%fireFrames)/fireFrames)), 0f ));
		animateTrans.setTransform(t3d);
		/* kill half way through the animation */
		if( currentAnimationFrame==fireFrames/2 ) {
			Transform3D pos= new Transform3D();
			edgeOfSpear.getLocalToVworld(pos);
			Vector3f v= new Vector3f();
			pos.get(v);
			for(int i=0;i<room.numChildren();i++) {
				try {
					if(((FileObject)room.getChild(i)).checkXZIntersect(v) && v.y<((FileObject)room.getChild(i)).getHeight()) ((FileObject)room.getChild(i)).hit( damage );
				} catch( Exception e ) {}
			}
		}
	}

	public void secondaryFireThisWeapon() {}

	public void endAnimate() {
		t3d= new Transform3D();
		animateTrans.getTransform(t3d);
		t3d.setTranslation( new Vector3f( 0, -.08f*currentAnimationFrame, 0 ));
		animateTrans.setTransform(t3d);
	}
}
