/*
    BFM, the Brutal File Manager. A 3D fps filemanager
    Copyright (C) 2003  Henrik Johansson and Daniel Forchheimer

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import java.awt.*;
import com.sun.j3d.utils.geometry.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import java.io.*;

public class Wall extends TransformGroup {
	private File directory, currentDir;
	private String textString;
	private Material wallMaterial;
	private Appearance wallAppearance;
	private Color3f doorColor= new Color3f( .2f, .2f, .4f),
	                hiddenDoorColor= new Color3f(.02f, .32f, .52f),
	                forbiddenDoorColor= new Color3f(.4f,.2f,.2f),
									forbiddenHiddenDoorColor= new Color3f(.6f,.3f,.3f),
	                noDoorColor= new Color3f(.02f, .32f, .52f),
	                parentDoorColor= new Color3f(.2f, .4f, .2f);
	
	public Wall( File directory, File currentDir, int length, int height ) {
		this.currentDir=currentDir;
		this.directory= directory;
		createWall(length,height);
		createText();
	}
		
	public Wall(int length,int height ) {
		createWall(length,height);
	}
		
	private void createText() {
		Transform3D t3d= new Transform3D();
		textString= directory.getName();
		try {
			if( currentDir.getParent().equals( directory.getPath() )) textString="..";
		} catch( Exception jaghatarfysik ) {}

		t3d.rotY(Math.PI);	
		t3d.setScale(4);
		t3d.setTranslation(new Vector3f((float)(textString.length()*70/512),0,-0.2f));
		TransformGroup textMoveTrans= new TransformGroup(t3d);
		try {
			Text2D text= new Text2D(textString,new Color3f(1,1,1),"Arial",40,Font.PLAIN);
			textMoveTrans.addChild(text);
		} catch( Exception ojojoj ) {}
		addChild(textMoveTrans);
	}

	private void createWall(int wallLength, int wallHeight) {
		GeometryArray g;
		Shape3D s;
		g= new QuadArray(4,QuadArray.COORDINATES|QuadArray.NORMALS);

		Vector3f[] v= new Vector3f[4];
		v[0]=new Vector3f(0,0,1f);
		v[1]=new Vector3f(0,0,1f);
		v[2]=new Vector3f(0,0,1f);
		v[3]=new Vector3f(0,0,1f);
		g.setNormals(0,v);
		
		float l= wallLength/2;
		float h= wallHeight/2;
		Point3f[] tt=new Point3f[4];
		tt[0]=new Point3f(-l,h,0);
		tt[1]=new Point3f(l,h,0);
		tt[2]=new Point3f(l,-h,0);
		tt[3]=new Point3f(-l,-h,0);
		g.setCoordinates(0,tt);

		s= new Shape3D(g,setWallAppearance());
		addChild(s);
	}

	private Appearance setWallAppearance() {
		Color3f wallColor;
		
		/* make dir's color depend on read-flag, even while safe-mode */
		if(directory!=null) {
			if( directory.canRead() && !directory.isHidden() ) wallColor= doorColor;
			else if( directory.canRead() && directory.isHidden() ) wallColor= hiddenDoorColor;
			else wallColor= forbiddenDoorColor;
			try {
				if( currentDir.getParent().equals( directory.getPath() )) wallColor= parentDoorColor;
			} catch( Exception gurka ) {}		
		} else wallColor= noDoorColor;

		wallAppearance= new Appearance();
		wallAppearance.setCapability(Appearance.ALLOW_MATERIAL_WRITE);
		wallMaterial= new Material();
		wallMaterial.setCapability(Material.ALLOW_COMPONENT_WRITE);
		wallMaterial.setEmissiveColor( wallColor );
		wallAppearance.setMaterial( wallMaterial );
		return wallAppearance;
	}

	public void update( String mode ) {
		if( directory!=null && !textString.equals( ".." ) && mode.equals( "Remove" )) {
			if( directory.canWrite() ) {
				if( directory.isHidden() ) wallMaterial.setEmissiveColor( hiddenDoorColor );
				else wallMaterial.setEmissiveColor( doorColor );
			} else {
				if( directory.isHidden() ) wallMaterial.setEmissiveColor( forbiddenHiddenDoorColor );
				else wallMaterial.setEmissiveColor( forbiddenDoorColor );
			}
		} else if( directory!=null && !textString.equals( ".." ) && ( mode.equals( "Select" ) || mode.equals( "Unselect" ))) {
			if( directory.canRead() ) {
				if( directory.isHidden() ) wallMaterial.setEmissiveColor( hiddenDoorColor );
				else wallMaterial.setEmissiveColor( doorColor );
			} else {
				if( directory.isHidden() ) wallMaterial.setEmissiveColor( forbiddenHiddenDoorColor );
				else wallMaterial.setEmissiveColor( forbiddenDoorColor );
			}
		}
	}

	public String getPath() {
 		if(directory!=null) return directory.getAbsolutePath();
 		return "";
	}
}
