/*
    BFM, the Brutal File Manager. A 3D fps filemanager
    Copyright (C) 2003  Henrik Johansson and Daniel Forchheimer

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.picking.*;

public abstract class Weapon extends BranchGroup {
	private TransformGroup root, v1, v2;
	protected Player player;
	protected boolean unlimitedFire, unlimitedSecondaryFire, repeatFire, repeatSecondaryFire, firing, stillFiring, autoSetAmmo= true;
	protected int ammo, currentAnimationFrame, animationFrames, fireFrames;
	protected Room room;
	protected float fireDelay, timer= 0;
	protected Transform3D t3d;

	public Weapon( Player player ) {
		this.player= player;
		setCapability( ALLOW_DETACH );
		setCapability( ALLOW_CHILDREN_WRITE );
		setCapability( ALLOW_CHILDREN_EXTEND );
		addChild( createShape() );
		v1= new TransformGroup();
		Transform3D t3d= new Transform3D();
		t3d.setTranslation( new Vector3f( 0, 0, -1f ));
		v2= new TransformGroup(t3d);
		v1.setCapability(ALLOW_LOCAL_TO_VWORLD_READ);
		v2.setCapability(ALLOW_LOCAL_TO_VWORLD_READ);
		v2.addChild( v1 );
		addChild( v2 );
	}

	abstract void initializeAnimate();
	abstract TransformGroup getCrosshair();
	abstract TransformGroup createShape();
	abstract void fireThisWeapon();
	abstract void secondaryFireThisWeapon();
	abstract void endAnimate();

	public float getTimer() { return timer; }
	public boolean isFiring() { return firing; }
	public boolean getRepeatFire() { return repeatFire; }
	public boolean getRepeatSecondaryFire() { return repeatSecondaryFire; }

	public boolean initialize() {
		currentAnimationFrame++;
		initializeAnimate();
		if( currentAnimationFrame==animationFrames ) {
			player.getHud().setCrosshair( getCrosshair() );
			/*if( unlimitedFire ) player.getHud().setAmmoText( "-" );
			else player.getHud().setAmmoText( Integer.toString( ammo ));*/
			currentAnimationFrame= 0;
			return true;
		} else return false;
	}

	public void fire( Room room, float delay, boolean stillFiring ) {
		currentAnimationFrame++;
		this.stillFiring= stillFiring;
		if( (currentAnimationFrame==1 && ammo>0) || unlimitedFire ) {
			firing= true;
			this.room= room;
			if( !unlimitedFire && ammo>0 && autoSetAmmo ) {
				ammo--;
				player.getHud().setAmmoText( Integer.toString( ammo ));
			}
		}
		/* delay after firing */
		if( currentAnimationFrame>fireFrames ) {
			timer+= delay;
			if( timer>= fireDelay ) finishFiring();
		} else if( unlimitedFire || ammo>0 || ( ammo==0 && firing )) fireThisWeapon();
	}

	public void finishFiring() {
		currentAnimationFrame= 0;
		firing= false;
		timer= 0;	
	}

	public void secondaryFire(Room room) {
		this.room= room;
		if( (!unlimitedSecondaryFire && ammo>0) || unlimitedSecondaryFire ) {
			if( !unlimitedSecondaryFire ) {
				ammo--;
				player.getHud().setAmmoText( Integer.toString( ammo ));
			}
			secondaryFireThisWeapon();
		}
	}

	public boolean end() {
		currentAnimationFrame++;
		if( currentAnimationFrame==1 ) {
			player.getHud().setCrosshair( new TransformGroup() );
			player.getHud().setAmmoText( "" );
		}
		endAnimate();
		if( currentAnimationFrame==animationFrames) {
			currentAnimationFrame= 0;
			return true;
		} else return false;
	}

	public void updateAimingAt( Room room ) {
		try {
			Transform3D pos1= new Transform3D();
			v2.getLocalToVworld(pos1);
			Vector3d v= new Vector3d();
			pos1.get(v);
		
			Transform3D pos0= new Transform3D();
			v1.getLocalToVworld(pos0);
			Vector3d v0= new Vector3d();
			pos0.get(v0);

			Vector3d dir= new Vector3d();
			dir.sub(v0,v);
		
			PickTool pt= new PickTool(room);
			Point3d p= new Point3d(new Point3f(v0));
		
			pt.setShapeRay(p,dir);
			pt.setMode(PickTool.GEOMETRY);
			PickResult pr= pt.pickClosest();
			player.getHud().setAimingAtText( ((FileObject)pr.getNode(PickResult.BRANCH_GROUP)).getFile().getName() );
		} catch( Exception e ) { player.getHud().setAimingAtText( "" ); }
	}
}
